@@ -1,6 +1,6 @@
 /*
  *  agShell
- *  $Id: agShell.c,v 1.11 2001/05/28 17:21:32 bkorb Exp $
+ *  $Id: agShell.c,v 1.15 2001/10/27 17:34:59 bkorb Exp $
  *  Manage a server shell process
  */
 
@@ -41,14 +41,13 @@
 /*
  *  Dual pipe opening of a child process
  */
-STATIC tpChar  defArgs[]  = { (char*)NULL, (char*)NULL };
 STATIC tpfPair serverPair = { (FILE*)NULL, (FILE*)NULL };
 STATIC pid_t   serverId   = NULLPROCESS;
 STATIC tpChar  pCurDir    = (char*)NULL;
 STATIC ag_bool errClose   = AG_FALSE;
 
 tSCC   zTxtBlock[] = "Text Block";
-tSCC   zCmdFmt[]   = "\\cd %s\n%s\n\necho\necho %s\n";
+tSCC   zCmdFmt[]   = "cd %s\n%s\n\necho\necho %s\n";
 
 const char* pzLastCmd = (const char*)NULL;
 
@@ -64,8 +63,18 @@ closeServer( void )
 
     (void)kill( serverId, SIGKILL );
     serverId = NULLPROCESS;
-    (void)fclose( serverPair.pfRead );
-    (void)fclose( serverPair.pfWrite );
+
+    /*
+     *  This guard should not be necessary.  However, sometimes someone
+     *  holds an allocation pthread lock when a seg fault occurrs.  fclose
+     *  needs that lock, so we hang waiting for it.  Oops.  So, when we
+     *  are aborting, we just let the OS close these file descriptors.
+     */
+    if (procState != PROC_STATE_ABORTING) {
+        (void)fclose( serverPair.pfRead );
+        (void)fclose( serverPair.pfWrite );
+    }
+
     serverPair.pfRead = serverPair.pfWrite = (FILE*)NULL;
 }
 
@@ -120,11 +129,14 @@ serverSetup( void )
     errClose = AG_FALSE;
 
     {
-        tSCC zTrap[] = "for f in 1 2 5 6 7 13 14\n"
-                       "do trap \"echo trapped on $f >&2\" $f ; done";
+        tSCC zTrap[] =
+            "for f in 1 2 5 6 7 13 14\n"
+            "do trap \"echo trapped on $f >&2\" $f 2>/dev/null\n"
+            "done\n" "unalias cd 2>/dev/null >&2\n" "AG_pid=%d\n";
         char* pz;
         pzLastCmd = zTrap;
-        fprintf( serverPair.pfWrite, zCmdFmt, pCurDir, pzLastCmd, zShDone );
+        fprintf( serverPair.pfWrite, zCmdFmt, pCurDir,
+                 pzLastCmd, zShDone, getpid() );
         (void)fflush( serverPair.pfWrite );
         pz = loadData( serverPair.pfRead );
 #if defined( DEBUG ) && defined( VALUE_OPT_SHOW_SHELL )
@@ -136,8 +148,8 @@ serverSetup( void )
     if (HAVE_OPT( SHOW_SHELL )) {
         tSCC zSetup[] = "set -x\n"
                         "exec 2> /dev/tty\n"
-                        "trap >&2\n"
-                        "echo server setup done >&2\n";
+                        "trap\n"
+                        "echo server setup done\n";
         char* pz;
 
         fputs( "Server traps set\n", stderr );
@@ -145,8 +157,7 @@ serverSetup( void )
         fprintf( serverPair.pfWrite, zCmdFmt, pCurDir, pzLastCmd, zShDone );
         (void)fflush( serverPair.pfWrite );
         pz = loadData( serverPair.pfRead );
-        if (HAVE_OPT( SHOW_SHELL ))
-            fprintf( stderr, "Trap state:\n%s\n", pz );
+        fprintf( stderr, "Trap state:\n%s\n", pz );
 #endif
         AGFREE( (void*)pz );
     }
@@ -154,7 +165,6 @@ serverSetup( void )
 }
 
 
-
 /*
  *  chainOpen
  *
@@ -166,18 +176,18 @@ serverSetup( void )
  */
     EXPORT int
 chainOpen( int       stdinFd,
-           tpChar*   ppArgs,
+           tCC**     ppArgs,
            pid_t*    pChild  )
 {
     tFdPair   stdoutPair = { -1, -1 };
     pid_t     chId;
-    char*     pzShell;
+    tCC*      pzShell;
 
     /*
      *  If we did not get an arg list, use the default
      */
-    if (ppArgs == (tpChar*)NULL)
-        ppArgs = defArgs;
+    if (ppArgs == (tCC**)NULL)
+        ppArgs = serverArgs;
 
     /*
      *  If the arg list does not have a program,
@@ -207,7 +217,7 @@ chainOpen( int       stdinFd,
 #if defined( DEBUG ) && defined( VALUE_OPT_SHOW_SHELL )
     if (HAVE_OPT( SHOW_SHELL )) {
         tSCC   zPath[] = "PATH";
-        char*  pzPath  = getenv( zPath );
+        tCC*   pzPath  = getenv( zPath );
 
         if (pzPath == (char*)NULL)
             pzPath = pzShell;
@@ -271,7 +281,7 @@ chainOpen( int       stdinFd,
      */
     setvbuf( stdout, (char*)NULL, _IONBF, 0 );
 
-    execvp( pzShell, ppArgs );
+    execvp( (char*)pzShell, (char**)ppArgs );
     fprintf( stderr, "Error %d:  Could not execvp( '%s', ... ):  %s\n",
              errno, pzShell, strerror( errno ));
     AG_ABEND;
@@ -289,7 +299,7 @@ chainOpen( int       stdinFd,
  *  The return value is the process id of the created process.
  */
     EXPORT pid_t
-openServer( tFdPair* pPair, tpChar* ppArgs )
+openServer( tFdPair* pPair, tCC** ppArgs )
 {
     pid_t     chId;
 
@@ -316,7 +326,7 @@ openServer( tFdPair* pPair, tpChar* ppArgs )
  *  into file pointers instead.
  */
     EXPORT pid_t
-openServerFP( tpfPair* pfPair, tpChar* ppArgs )
+openServerFP( tpfPair* pfPair, tCC** ppArgs )
 {
     tFdPair   fdPair;
     pid_t     chId = openServer( &fdPair, ppArgs );
@@ -431,7 +441,7 @@ runShell( const char*  pzCmd )
      *  THEN try to start it.
      */
     if (serverId == NULLPROCESS) {
-        serverId = openServerFP( &serverPair, defArgs );
+        serverId = openServerFP( &serverPair, serverArgs );
         if (serverId > 0)
             serverSetup();
     }
@@ -490,5 +500,6 @@ runShell( const char*  pzCmd )
 /*
  * Local Variables:
  * c-file-style: "stroustrup"
+ * indent-tabs-mode: nil
  * End:
  * end of agShell.c */
