@@ -1,7 +1,7 @@
 
 /*
  *  agUtils.c
- *  $Id: agUtils.c,v 1.12 2001/05/19 22:06:22 bkorb Exp $
+ *  $Id: agUtils.c,v 1.16 2001/10/27 17:34:59 bkorb Exp $
  *  This is the main routine for autogen.
  */
 
@@ -32,20 +32,29 @@
 
 #include "autogen.h"
 
+#if defined( HAVE_POSIX_SYSINFO )
+#  include <sys/systeminfo.h>
+
+#elif defined( HAVE_UNAME_SYSCALL )
+#  include <sys/utsname.h>
+
+#endif
+
+STATIC void addSysEnv( char* pzEnvName );
+
 #ifdef MEMDEBUG
 #  include "snprintfv/mem.h"
    static void finalMemCheck( void );
 #endif
 
-STATIC const char* skipQuote( const char* pzQte );
-
+STATIC tCC* skipQuote( tCC* pzQte );
 
 #ifndef HAVE_STRLCPY
     size_t
-strlcpy( char* dest, const char* src, size_t n )
+strlcpy( char* dest, tCC* src, size_t n )
 {
     char* pz = dest;
-    const char* ps = src;
+    tCC* ps = src;
     size_t sz = 0;
 
     for (;;) {
@@ -184,11 +193,82 @@ doOptions( int arg_ct, char** arg_vec )
             putenv( pz );
         } while (--ct > 0);
     }
+
+    {
+        char z[ 128 ] = "__autogen__";
+#if defined( HAVE_POSIX_SYSINFO )
+        static const int nm[] = {
+            SI_SYSNAME, SI_HOSTNAME, SI_ARCHITECTURE, SI_HW_PROVIDER,
+#ifdef      SI_PLATFORM
+            SI_PLATFORM,
+#endif
+            SI_MACHINE };
+        int ix;
+        long sz;
+        char* pz = z+2;
+
+        addSysEnv( z );
+        for (ix = 0; ix < sizeof(nm)/sizeof(nm[0]); ix++) {
+            sz = sysinfo( nm[ix], z+2, sizeof( z ) - 2);
+            if (sz > 0) {
+                sz += 2;
+                while (z[sz-1] == NUL)  sz--;
+                strcpy( z + sz, "__" );
+                addSysEnv( z );
+            }
+        }
+
+#elif defined( HAVE_UNAME_SYSCALL )
+        struct utsname unm;
+
+        addSysEnv( z );
+        if (uname( &unm ) != 0) {
+            fprintf( stderr, "Error %d (%s) making uname(2) call\n",
+                     errno, strerror( errno ));
+            exit( EXIT_FAILURE );
+        }
+
+        sprintf( z+2, "%s__", unm.sysname );
+        addSysEnv( z );
+
+        sprintf( z+2, "%s__", unm.machine );
+        addSysEnv( z );
+
+        sprintf( z+2, "%s__", unm.nodename );
+        addSysEnv( z );
+#else
+
+        addSysEnv( z );
+#endif
+    }
 }
 
 
-    EXPORT const char*
-getDefine( const char* pzDefName )
+    STATIC void
+addSysEnv( char* pzEnvName )
+{
+    tSCC zFmt[] = "%s=1";
+    int i = 2;
+
+    for (;;) {
+        if (isupper( pzEnvName[i] ))
+            pzEnvName[i] = tolower( pzEnvName[i] );
+        else if (! isalnum( pzEnvName[i] ))
+            pzEnvName[i] = '_';
+
+        if (pzEnvName[ ++i ] == NUL)
+            break;
+    }
+    if (getenv( pzEnvName ) == NULL) {
+        if (OPT_VALUE_TRACE > TRACE_NOTHING)
+            fprintf( stderr, "Adding ``%s'' to environment\n", pzEnvName );
+        putenv( asprintf( zFmt, pzEnvName ));
+    }
+}
+
+
+    EXPORT tCC*
+getDefine( tCC* pzDefName )
 {
     char**  ppz;
     int     ct;
@@ -217,7 +297,7 @@ getDefine( const char* pzDefName )
 
 
     EXPORT unsigned int
-doEscapeChar( const char* pzIn, char* pRes )
+doEscapeChar( tCC* pzIn, char* pRes )
 {
     unsigned int  res = 1;
 
@@ -385,8 +465,8 @@ spanQuote( char* pzQte )
  *  The quote character is whatever character the argument
  *  is pointing at when this procedure is called.
  */
-    STATIC const char*
-skipQuote( const char* pzQte )
+    STATIC tCC*
+skipQuote( tCC* pzQte )
 {
     char  q = *pzQte++;        /*  Save the quote character type */
 
@@ -430,8 +510,8 @@ skipQuote( const char* pzQte )
 }
 
 
-    EXPORT const char*
-skipScheme( const char* pzSrc,  const char* pzEnd )
+    EXPORT tCC*
+skipScheme( tCC* pzSrc,  tCC* pzEnd )
 {
     int  level = 0;
 
@@ -455,10 +535,10 @@ skipScheme( const char* pzSrc,  const char* pzEnd )
 }
 
 
-    EXPORT const char*
-skipExpression( const char* pzSrc, size_t len )
+    EXPORT tCC*
+skipExpression( tCC* pzSrc, size_t len )
 {
-    const char* pzEnd = pzSrc + len;
+    tCC* pzEnd = pzSrc + len;
 
  guess_again:
 
@@ -468,7 +548,7 @@ skipExpression( const char* pzSrc, size_t len )
     switch (*pzSrc) {
     case ';':
         pzSrc = strchr( pzSrc, '\n' );
-        if (pzSrc == (const char*)NULL)
+        if (pzSrc == (tCC*)NULL)
             return pzEnd;
         goto guess_again;
 
@@ -494,7 +574,7 @@ skipExpression( const char* pzSrc, size_t len )
 
 
     void*
-ag_alloc( size_t sz, const char* pzWhat )
+ag_alloc( size_t sz, tCC* pzWhat )
 {
     void* p = malloc( sz );
     if ((p == (void*)NULL) && (pzWhat != NULL)) {
@@ -506,7 +586,7 @@ ag_alloc( size_t sz, const char* pzWhat )
 
 
     void*
-ag_realloc( void* p, size_t sz, const char* pzWhat )
+ag_realloc( void* p, size_t sz, tCC* pzWhat )
 {
     void* np = p ? realloc( p, sz ) : malloc( sz );
     if (np == (void*)NULL) {
@@ -523,7 +603,7 @@ ag_realloc( void* p, size_t sz, const char* pzWhat )
 
 
     char*
-ag_strdup( const char* pz, const char* pzWhat )
+ag_strdup( tCC* pz, tCC* pzWhat )
 {
     char*   pzRes;
     size_t  len = strlen( pz )+1;
@@ -553,7 +633,7 @@ STATIC tMemMgmt memHead = { &memHead, &memHead, (char*)NULL, "ROOT" };
 
 
     void*
-ag_alloc( size_t sz, const char* pzWhat, const char* pz )
+ag_alloc( size_t sz, tCC* pzWhat, tCC* pz )
 {
     size_t    asz = sz + sizeof( tMemMgmt ) + CHECK_CT + SPARE;
     tMemMgmt* p = (tMemMgmt*)malloc( asz & ~0x3F );
@@ -583,7 +663,7 @@ ag_alloc( size_t sz, const char* pzWhat, const char* pz )
 
 
     void*
-ag_realloc( void* p, size_t sz, const char* pzWhat, const char* pz )
+ag_realloc( void* p, size_t sz, tCC* pzWhat, tCC* pz )
 {
     size_t      asz = sz + sizeof( tMemMgmt ) + CHECK_CT + SPARE;
     tMemMgmt*   np  = ((tMemMgmt*)p)-1;
@@ -660,7 +740,7 @@ ag_free( void* p )
 }
 
 
-    static void
+    STATIC void
 finalMemCheck( void )
 {
     tMemMgmt*  pMM = memHead.pNext;
@@ -676,7 +756,7 @@ finalMemCheck( void )
 
 
     char*
-ag_strdup( const char* pz, const char* pzDupFrom, const char* pzWhat )
+ag_strdup( tCC* pz, tCC* pzDupFrom, tCC* pzWhat )
 {
     char*   pzRes;
     size_t  len = strlen( pz )+1;
@@ -702,5 +782,6 @@ ag_strdup( const char* pz, const char* pzDupFrom, const char* pzWhat )
 /*
  * Local Variables:
  * c-file-style: "stroustrup"
+ * indent-tabs-mode: nil
  * End:
  * end of agUtils.c */
