@@ -1,7 +1,7 @@
 
 /*
  *  autogen.c
- *  $Id: autogen.c,v 1.15 2001/03/21 15:40:22 bkorb Exp $
+ *  $Id: autogen.c,v 1.20 2001/10/27 17:34:59 bkorb Exp $
  *  This is the main routine for autogen.
  */
 
@@ -89,8 +89,11 @@ main( int    argc,
       char** argv )
 {
     pfTrace = stderr;
-    if (sigsetjmp( abendJumpEnv, 0 ) != 0)
-        exit( EXIT_FAILURE );
+    {
+        int signo = sigsetjmp( abendJumpEnv, 0 );
+        if (signo != 0)
+            exit( signo + 128 );
+    }
 
     signalSetup();
 
@@ -100,34 +103,6 @@ main( int    argc,
 }
 
 
-    STATIC void
-abendSignal( int sig )
-{
-    tSCC zErr[] = "AutoGen aborting on signal %d (%s)\n";
-    tSCC zAt[]  = "processing template %s\n"
-                  "            on line %d\n"
-                  "       for function %s (%d)\n";
-
-    fprintf( stderr, zErr, sig, strsignal( sig ));
-
-    /*
-     *  IF there is a current template, then we should report where we are
-     *  so that the template writer knows where to look for their problem.
-     */
-    if (pCurTemplate != (tTemplate*)NULL) {
-        int f;
-        assert( pCurMacro != NULL );
-        f = (pCurMacro->funcCode > FUNC_CT)
-                ? FTYP_SELECT : pCurMacro->funcCode;
-        fprintf( stderr, zAt, pCurTemplate->pzFileName, pCurMacro->lineNo,
-                 apzFuncNames[ f ], pCurMacro->funcCode );
-    }
-
-    procState = PROC_STATE_ABORTING;
-    siglongjmp( abendJumpEnv, sig );
-}
-
-
     STATIC void
 doneCheck( void )
 {
@@ -171,19 +146,56 @@ doneCheck( void )
     case PROC_STATE_OPTIONS:
     case PROC_STATE_DONE:
         return;
-
-    default: break;
     }
 
     _exit( EXIT_FAILURE );
 }
 
 
+    STATIC void
+ignoreSignal( int sig )
+{
+    return;
+}
+
+
+    STATIC void
+abendSignal( int sig )
+{
+    tSCC zErr[] = "AutoGen aborting on signal %d (%s)\n";
+    tSCC zAt[]  = "processing template %s\n"
+                  "            on line %d\n"
+                  "       for function %s (%d)\n";
+
+    fprintf( stderr, zErr, sig, strsignal( sig ));
+    fflush( stderr );
+    fflush( stdout );
+    if (pfTrace != stderr )
+        fflush( pfTrace );
+
+    /*
+     *  IF there is a current template, then we should report where we are
+     *  so that the template writer knows where to look for their problem.
+     */
+    if (pCurTemplate != (tTemplate*)NULL) {
+        int f;
+        assert( pCurMacro != NULL );
+        f = (pCurMacro->funcCode > FUNC_CT)
+                ? FTYP_SELECT : pCurMacro->funcCode;
+        fprintf( stderr, zAt, pCurTemplate->pzFileName, pCurMacro->lineNo,
+                 apzFuncNames[ f ], pCurMacro->funcCode );
+    }
+
+    procState = PROC_STATE_ABORTING;
+    siglongjmp( abendJumpEnv, sig );
+}
+
+
     STATIC void
 signalSetup( void )
 {
     struct sigaction  sa;
-    int    sigNo = 0;
+    int    sigNo = 1;
 
     atexit( doneCheck );
 
@@ -196,19 +208,39 @@ signalSetup( void )
     snv_free    = (void(*)SNV_PARAMS((snv_pointer)))ag_free;
 #endif
 
-    sa.sa_handler = abendSignal;
     sa.sa_flags   = 0;
     sigemptyset( &sa.sa_mask );
 
     do  {
-        sigaction( ++sigNo,  &sa, (struct sigaction*)NULL );
-    } while (sigNo < NSIG);
+        switch (sigNo) {
+            /*
+             *  Signal handling for SIGCHLD needs to be ignored.  Do *NOT* use
+             *  SIG_IGN to do this.  That gets inherited by programs that need
+             *  to be able to use wait(2) calls.  At the same time, we don't
+             *  want our children to become zombies.  We may run out of zombie
+             *  space.  Therefore, set the handler to an empty procedure.
+             *  POSIX oversight.  Fixed for next POSIX rev.
+             */
+        case SIGCHLD:
+#ifdef SIGWINCH
+        case SIGWINCH:
+#endif
+        case SIGSTOP:  /* suspended */
+            sa.sa_handler = ignoreSignal;
+            break;
 
-    sa.sa_handler = SIG_DFL;
+        default:
+            sa.sa_handler = abendSignal;
+        }
+        sigaction( sigNo,  &sa, (struct sigaction*)NULL );
+    } while (++sigNo < NSIG);
+
+    sa.sa_handler = ignoreSignal;
     sigaction( SIGCHLD,  &sa, (struct sigaction*)NULL );
 }
 /*
  * Local Variables:
  * c-file-style: "stroustrup"
+ * indent-tabs-mode: nil
  * End:
  * end of autogen.c */
