@@ -1,7 +1,7 @@
 
 /*
  *  autogen.h
- *  $Id: autogen.h,v 1.21 2001/04/30 03:38:32 bkorb Exp $
+ *  $Id: autogen.h,v 1.26 2001/08/26 03:37:17 bkorb Exp $
  *  Global header file for AutoGen
  */
 
@@ -86,9 +86,6 @@ typedef struct template_lib_marker tTlibMark;
  */
 typedef tMacro* (tLoadProc)( tTemplate*, tMacro*, const char** ppzScan );
 typedef tLoadProc* tpLoadProc;
-#define MAKE_LOAD_PROC( n ) \
-    tMacro* \
-mLoad_ ## n ( tTemplate*  pT,  tMacro*  pMac, const char**  ppzScan )
 
 /*
  *  Procedure for handling a template function
@@ -96,9 +93,6 @@ mLoad_ ## n ( tTemplate*  pT,  tMacro*  pMac, const char**  ppzScan )
  */
 typedef tMacro* (tHdlrProc)( tTemplate*, tMacro* );
 typedef tHdlrProc* tpHdlrProc;
-#define MAKE_HANDLER_PROC( n ) \
-    tMacro* \
-mFunc_ ## n ( tTemplate* pT,  tMacro* pMac )
 
 /*
  *  This must be included after the function prototypes
@@ -211,7 +205,13 @@ struct outSpec {
     char        zSuffix[ 1 ];
 };
 
+#define FPF_FREE       0x0001  /* free the fp structure   */
+#define FPF_UNLINK     0x0002  /* unlink file (temp file) */
+#define FPF_NOUNLINK   0x0004  /* do not unlink file      */
+#define FPF_STATIC_NM  0x0008  /* name statically alloced */
+
 struct fpStack {
+	int         flags;
     tFpStack*   pPrev;
     FILE*       pFile;
     char*       pzOutName;
@@ -260,7 +260,7 @@ MODE tTemplate*  pNamedTplList    VALUE( (tTemplate*)NULL );
 /*
  *  Template Processing Globals
  */
-MODE const char* pzCurSfx         VALUE( (char*)NULL );
+MODE tCC*        pzCurSfx         VALUE( (char*)NULL );
 MODE time_t      outTime          VALUE( 0 );
 MODE tFpStack*   pCurFp           VALUE( (tFpStack*)NULL );
 MODE tOutSpec*   pOutSpecList     VALUE( (tOutSpec*)NULL );
@@ -270,6 +270,8 @@ MODE off_t       curStartOff      VALUE( 0 );
 MODE tForInfo    forInfo          VALUE( { 0 } );
 MODE FILE*       pfTrace          VALUE( (FILE*)NULL );
 
+MODE tCC*        serverArgs[2]    VALUE( { (char*)NULL } );
+
 /*
  *  AutoGen definiton and template context
  *
@@ -336,9 +338,6 @@ MKSTRING( FileLine,  "\tfrom %s line %d\n" );
 MKSTRING( GiveUp,    "Giving up in %s line %d\n" );
 #endif
 
-typedef char t_scribble_buf[ 4096 ];
-MODE t_scribble_buf zScribble VALUE( "" );
-
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *
  *  MEMORY DEBUGGING
@@ -403,6 +402,24 @@ struct mem_mgmt {
 size_t strlcpy( char* dest, const char* src, size_t n );
 #endif
 
+static inline char* ag_scm2zchars( SCM s, tCC* type )
+{
+    if (! gh_string_p( s )) {
+        tSCC zNotStr[] = "ERROR: %s is not a string\n";
+        fprintf( stderr, zNotStr, type );
+        LOAD_ABORT( pCurTemplate, pCurMacro, zNotStr+14 );
+    }
+
+    if (SCM_SUBSTRP(s))
+        s = scm_makfromstr( SCM_ROCHARS(s), SCM_ROLENGTH(s), 0 );
+    return SCM_CHARS(s);
+}
+
 #include "proto.h"
 #endif /* AUTOGEN_HDR */
-/* end of autogen.h */
+/*
+ * Local Variables:
+ * c-file-style: "Stroustrup"
+ * indent-tabs-mode: nil
+ * End:
+ * end of autogen.h */
