@@ -11,7 +11,7 @@ static char yyrcsid[] = "$Id: skeleton.c,v 1.4 1993/12/21 18:45:32 jtc Exp $";
 #define YYPREFIX "yy"
 #line 2 "defParse.y"
 /*
- *  $Id: defParse.y,v 1.8 2001/04/30 03:38:32 bkorb Exp $
+ *  $Id: defParse.y,v 1.9 2001/07/15 23:05:48 bkorb Exp $
  *  Parse rules for the macro definitions
  */
 
@@ -49,7 +49,7 @@ extern YYSTYPE appendList( YYSTYPE, YYSTYPE );
 extern YYSTYPE findPlace( YYSTYPE, YYSTYPE );
 extern YYSTYPE identify( YYSTYPE );
 
-extern int yyparse( void );;
+extern int yyparse( void );
 
 #line 55 "y.tab.c"
 #define TK_AUTOGEN 257
