@@ -1,7 +1,7 @@
 /*  -*- Mode: C -*-
  *
  *  expFormat.c
- *  $Id: expFormat.c,v 1.36 2001/05/08 04:27:32 bkorb Exp $
+ *  $Id: expFormat.c,v 1.42 2001/09/22 04:11:13 bkorb Exp $
  *  This module implements formatting expression functions.
  */
 
@@ -140,21 +140,14 @@ tSCC zNil[]      = "";
     SCM
 ag_scm_dne( SCM prefix, SCM first )
 {
+    char     zScribble[ 128 ];
     char*    pzRes;
     tCC*     pzFirst = zNil;
     SCM      res;
-
-    if (! gh_string_p( prefix ))
-        return SCM_UNDEFINED;
-
-    {
-        int len = SCM_LENGTH( prefix );
-        if (len >= 128) {
-            fprintf( stderr, zPfxMsg, zPfxLen, 128 );
-            LOAD_ABORT( pCurTemplate, pCurMacro, zPfxLen );
-        }
-        memcpy( (void*)zScribble, (void*)SCM_CHARS( prefix ), len );
-        zScribble[ len ] = NUL;
+    tCC*     pzPrefix = ag_scm2zchars( prefix, "prefix" );
+    if (strlen( pzPrefix ) > 128 ) {
+        fprintf( stderr, zPfxMsg, zPfxLen, 128 );
+        LOAD_ABORT( pCurTemplate, pCurMacro, zPfxLen );
     }
 
     /*
@@ -167,20 +160,19 @@ ag_scm_dne( SCM prefix, SCM first )
             fprintf( stderr, zPfxMsg, zPfxLen, 128 );
             LOAD_ABORT( pCurTemplate, pCurMacro, zPfxLen );
         }
-        memcpy( (void*)(zScribble + 128), (void*)SCM_CHARS( first ), len );
-        zScribble[ len + 128 ] = NUL;
+
         pzFirst = asprintf( ENABLED_OPT( WRITABLE ) ? "%s\n" : zDne1,
-                            zScribble+128, zScribble );
+                            ag_scm2zchars( first, "first-prefix" ), pzPrefix );
     }
 
     {
         time_t    curTime = time( (time_t*)NULL );
         struct tm*  pTime = localtime( &curTime );
-        strftime( zScribble+128, 128, "%A %B %e, %Y at %r %Z", pTime );
+        strftime( zScribble, 128, "%A %B %e, %Y at %r %Z", pTime );
     }
 
     pzRes = asprintf( ENABLED_OPT( WRITABLE ) ? zDne2 : zDne,
-                      zScribble, pCurFp->pzOutName, zScribble+128,
+                      pzPrefix, pCurFp->pzOutName, zScribble,
                       OPT_ARG( DEFINITIONS ), pzTemplFileName, pzFirst );
 
     if (pzRes == (char*)NULL) {
@@ -217,14 +209,14 @@ ag_scm_dne( SCM prefix, SCM first )
  *  @end example
  *
  *  The first three arguments to this format are provided by the
- *  routine and are: The name of the template file, the line within
+ *  routine and are:  The name of the template file, the line within
  *  the template where the error was found, and the current output
  *  file name.
  *
  *  After displaying the message, the current output file is removed
  *  and autogen exits with the EXIT_FAILURE error code.  IF, however,
- *  the argument is the number 0 (zero), then processing continues
- *  with the next suffix.
+ *  the argument begins with the number 0 (zero), or the string is the
+ *  empty string, then processing continues with the next suffix.
 =*/
     SCM
 ag_scm_error( SCM res )
@@ -247,10 +239,10 @@ ag_scm_error( SCM res )
 
     case GH_TYPE_NUMBER:
     {
-        long val = gh_scm2long( res );
+        unsigned long val = gh_scm2ulong( res );
         if (val == 0)
             abort = PROBLEM;
-        snprintf( zNum, sizeof( zNum ), "%ld", val );
+        snprintf( zNum, sizeof( zNum ), "%d", val );
         pzMsg = zNum;
         break;
     }
@@ -264,7 +256,7 @@ ag_scm_error( SCM res )
         break;
 
     case GH_TYPE_STRING:
-        pzMsg  = SCM_CHARS( res );
+        pzMsg  = ag_scm2zchars( res, "error string" );
         msgLen = SCM_LENGTH( res );
         while (isspace( *pzMsg ) && (--msgLen > 0)) pzMsg++;
 
@@ -287,7 +279,7 @@ ag_scm_error( SCM res )
      *  IF there is a message,
      *  THEN print it.
      */
-    if (*pzMsg != '\0')
+    if (*pzMsg != NUL)
         fprintf( stderr, zFmt, (abort != PROBLEM) ? zErr : zWarn,
                  pCurTemplate->pzFileName, pCurMacro->lineNo,
                  pCurFp->pzOutName, pzMsg );
@@ -306,9 +298,10 @@ ag_scm_error( SCM res )
  * exparg: prefix, String for starting each output line
  *
  * doc:
- *  Emit a string that contains the GNU Public License.  It
- *  takes two arguments: @code{prefix} contains the string to start
- *  each output line, and
+ *
+ *  Emit a string that contains the GNU Public License.
+ *  It takes two arguments:
+ *  @code{prefix} contains the string to start each output line, and
  *  @code{prog_name} contains the name of the program the copyright is
  *  about.
  *
@@ -316,44 +309,34 @@ ag_scm_error( SCM res )
     SCM
 ag_scm_gpl( SCM prog_name, SCM prefix )
 {
-    char*     pzPfx;
-    char*     pzRes;
-    SCM       res;
-    int       len;
-
-    if (! (   gh_string_p( prog_name )
-           && gh_string_p( prefix )))
-        return SCM_UNDEFINED;
+    char*   pzPfx  = ag_scm2zchars( prefix, "GPL line prefix" );
+    char*   pzPrg  = ag_scm2zchars( prog_name, "program name" );
+    char*   pzRes;
+    SCM     res;
+    int     len;
 
     /*
      *  Get the addresses of the program name and prefix strings.
      *  Make sure they are reasonably sized (<256 for program name
      *  and <128 for a line prefix).  Copy them to the scratch buffer.
      */
-    len = SCM_LENGTH( prog_name );
-    if (len >= 256) {
+    if (SCM_LENGTH( prog_name ) >= 256) {
         fprintf( stderr, zPfxMsg, zProgLen, 256 );
         LOAD_ABORT( pCurTemplate, pCurMacro, zProgLen );
     }
-    memcpy( (void*)zScribble, (void*)SCM_CHARS( prog_name ), len );
-    zScribble[ len ] = NUL;
 
-    pzPfx = zScribble + len + 1;
-    len   = SCM_LENGTH( prefix );
-    if (len >= 128) {
+    if (SCM_LENGTH( prefix ) >= 128) {
         fprintf( stderr, zPfxMsg, zPfxLen, 128 );
         LOAD_ABORT( pCurTemplate, pCurMacro, zPfxLen );
     }
-    memcpy( (void*)pzPfx, (void*)SCM_CHARS( prefix ), len );
-    pzPfx[ len ] = NUL;
 
     /*
      *  Allocate-sprintf the result string, then put it in a new SCM.
      */
-    pzRes = asprintf( zGpl, zScribble, pzPfx );
+    pzRes = asprintf( zGpl, pzPrg, pzPfx );
 
     if (pzRes == (char*)NULL) {
-        fprintf( stderr, zAllocErr, pzProg, -1, "GPL string" );
+        fprintf( stderr, zAllocErr, pzPrg, -1, "GPL string" );
         LOAD_ABORT( pCurTemplate, pCurMacro, zFmtAlloc );
     }
 
@@ -373,67 +356,49 @@ ag_scm_gpl( SCM prog_name, SCM prefix )
  * exparg: prefix, String for starting each output line
  *
  * doc:
- *  Emit a string that contains the GNU Library Public License.  It
- *  takes three arguments: @code{prefix} contains the string to start
- *  each output line.  @code{owner} contains the copyright owner.
- *  @code{prog_name} contains the name of the program the copyright is
- *  about.
+ *
+ *  Emit a string that contains the GNU Library Public License.  It takes
+ *  three arguments:  @code{prefix} contains the string to start each output
+ *  line.  @code{owner} contains the copyright owner.  @code{prog_name}
+ *  contains the name of the program the copyright is about.
  *
 =*/
     SCM
 ag_scm_lgpl( SCM prog_name, SCM owner, SCM prefix )
 {
-    char*     pzPfx;
-    char*     pzOwner;
-    char*     pzRes;
-    SCM       res;
-    int       len;
-
-    if (! (   gh_string_p( prog_name )
-           && gh_string_p( owner )
-           && gh_string_p( prefix )))
-        return SCM_UNDEFINED;
+    char*   pzPfx   = ag_scm2zchars( prefix, "GPL line prefix" );
+    char*   pzPrg  = ag_scm2zchars( prog_name, "program name" );
+    char*   pzOwner = ag_scm2zchars( owner, "owner" );
+    char*   pzRes;
+    SCM     res;
 
     /*
      *  Get the addresses of the program name prefix and owner strings.
      *  Make sure they are reasonably sized (<256 for program name
      *  and <128 for a line prefix).  Copy them to the scratch buffer.
      */
-    len = SCM_LENGTH( prog_name );
-    if (len >= 256) {
+    if (SCM_LENGTH( prog_name ) >= 256) {
         fprintf( stderr, zPfxMsg, zProgLen, 256 );
         LOAD_ABORT( pCurTemplate, pCurMacro, zProgLen );
     }
-    memcpy( (void*)zScribble, (void*)SCM_CHARS( prog_name ), len );
-    zScribble[ len ] = NUL;
 
-    /* prefix */
-    pzPfx = zScribble + len + 1;
-    len   = SCM_LENGTH( prefix );
-    if (len >= 128) {
+    if (SCM_LENGTH( prefix ) >= 128) {
         fprintf( stderr, zPfxMsg, zPfxLen, 128 );
         LOAD_ABORT( pCurTemplate, pCurMacro, zPfxLen );
     }
-    memcpy( (void*)pzPfx, (void*)SCM_CHARS( prefix ), len );
-    pzPfx[ len ] = NUL;
 
-    /* owner */
-    pzOwner = pzPfx + len + 1;
-    len     = SCM_LENGTH( owner );
-    if (len >= 256) {
+    if (SCM_LENGTH( owner ) >= 256) {
         fprintf( stderr, zPfxMsg, zOwnLen, 256 );
         LOAD_ABORT( pCurTemplate, pCurMacro, zOwnLen );
     }
-    memcpy( (void*)pzOwner, (void*)SCM_CHARS( owner ), len );
-    pzOwner[ len ] = NUL;
 
     /*
      *  Allocate-sprintf the result string, then put it in a new SCM.
      */
-    pzRes = asprintf( zLgpl, zScribble, pzPfx, pzOwner );
+    pzRes = asprintf( zLgpl, pzPrg, pzPfx, pzOwner );
 
     if (pzRes == (char*)NULL) {
-        fprintf( stderr, zAllocErr, pzProg, -1, "LGPL string" );
+        fprintf( stderr, zAllocErr, pzPrg, -1, "LGPL string" );
         LOAD_ABORT( pCurTemplate, pCurMacro, zFmtAlloc );
     }
 
@@ -453,18 +418,20 @@ ag_scm_lgpl( SCM prog_name, SCM owner, SCM prefix )
  * exparg: prefix, String for starting each output line
  *
  * doc:
- *  Emit a string that contains the Free BSD Public License.  It
- *  takes three arguments: @code{prefix} contains the string to start
- *  each output line.  @code{owner} contains the copyright owner.
- *  @code{prog_name} contains the name of the program the copyright is
- *  about.
+ *
+ *  Emit a string that contains the Free BSD Public License.
+ *  It takes three arguments:
+ *  @code{prefix} contains the string to start each output line.
+ *  @code{owner} contains the copyright owner.
+ *  @code{prog_name} contains the name of the program the copyright is about.
  *
 =*/
     SCM
 ag_scm_bsd( SCM prog_name, SCM owner, SCM prefix )
 {
-    char*   pzPfx;
-    char*   pzOwner;
+    char*   pzPfx   = ag_scm2zchars( prefix, "GPL line prefix" );
+    char*   pzPrg  = ag_scm2zchars( prog_name, "program name" );
+    char*   pzOwner = ag_scm2zchars( owner, "owner" );
     char*   pzRes;
     SCM     res;
     int     len;
@@ -479,41 +446,28 @@ ag_scm_bsd( SCM prog_name, SCM owner, SCM prefix )
      *  Make sure they are reasonably sized (<256 for program name
      *  and <128 for a line prefix).  Copy them to the scratch buffer.
      */
-    len = SCM_LENGTH( prog_name );
-    if (len >= 256) {
+    if (SCM_LENGTH( prog_name ) >= 256) {
         fprintf( stderr, zPfxMsg, zProgLen, 256 );
         LOAD_ABORT( pCurTemplate, pCurMacro, zProgLen );
     }
-    memcpy( (void*)zScribble, (void*)SCM_CHARS( prog_name ), len );
-    zScribble[ len ] = NUL;
 
-    /* prefix */
-    pzPfx = zScribble + len + 1;
-    len   = SCM_LENGTH( prefix );
-    if (len >= 128) {
+    if (SCM_LENGTH( prefix ) >= 128) {
         fprintf( stderr, zPfxMsg, zPfxLen, 128 );
         LOAD_ABORT( pCurTemplate, pCurMacro, zPfxLen );
     }
-    memcpy( (void*)pzPfx, (void*)SCM_CHARS( prefix ), len );
-    pzPfx[ len ] = NUL;
 
-    /* owner */
-    pzOwner = pzPfx + len + 1;
-    len     = SCM_LENGTH( owner );
-    if (len >= 256) {
+    if (SCM_LENGTH( owner ) >= 256) {
         fprintf( stderr, zPfxMsg, zOwnLen, 256 );
         LOAD_ABORT( pCurTemplate, pCurMacro, zOwnLen );
     }
-    memcpy( (void*)pzOwner, (void*)SCM_CHARS( owner ), len );
-    pzOwner[ len ] = NUL;
 
     /*
      *  Allocate-sprintf the result string, then put it in a new SCM.
      */
-    pzRes = asprintf( zBsd, zScribble, pzPfx, pzOwner );
+    pzRes = asprintf( zBsd, pzPrg, pzPfx, pzOwner );
 
     if (pzRes == (char*)NULL) {
-        fprintf( stderr, zAllocErr, pzProg, -1, "BSD string" );
+        fprintf( stderr, zAllocErr, pzPrg, -1, "BSD string" );
         LOAD_ABORT( pCurTemplate, pCurMacro, zFmtAlloc );
     }
 
@@ -544,18 +498,16 @@ ag_scm_bsd( SCM prog_name, SCM owner, SCM prefix )
     SCM
 ag_scm_license( SCM license, SCM prog_name, SCM owner, SCM prefix )
 {
+    char*   pzPfx   = ag_scm2zchars( prefix, "GPL line prefix" );
+    char*   pzPrg  = ag_scm2zchars( prog_name, "program name" );
+    char*   pzOwner = ag_scm2zchars( owner, "owner" );
     static tMapInfo mi = { NULL, 0, 0, NULL };
 
-    char*     pzPfx;
-    char*     pzOwner;
     char*     pzRes;
     SCM       res;
     int       len;
 
-    if (! (   gh_string_p( license )
-           && gh_string_p( prog_name )
-           && gh_string_p( owner )
-           && gh_string_p( prefix )))
+    if (! gh_string_p( license ))
         return SCM_UNDEFINED;
 
     {
@@ -596,40 +548,25 @@ ag_scm_license( SCM license, SCM prog_name, SCM owner, SCM prefix )
      *  Make sure they are reasonably sized (<256 for program name
      *  and <128 for a line prefix).  Copy them to the scratch buffer.
      */
-    len = SCM_LENGTH( prog_name );
-    if (len >= 256) {
+    if (SCM_LENGTH( prog_name ) >= 256) {
         fprintf( stderr, zPfxMsg, zProgLen, 256 );
         LOAD_ABORT( pCurTemplate, pCurMacro, zProgLen );
     }
-    memcpy( (void*)zScribble, (void*)SCM_CHARS( prog_name ), len );
-    zScribble[ len ] = NUL;
-
-    /* prefix */
-    pzPfx = zScribble + len + 1;
-    len   = SCM_LENGTH( prefix );
-    if (len >= 128) {
+    if (SCM_LENGTH( prefix ) >= 128) {
         fprintf( stderr, zPfxMsg, zPfxLen, 128 );
         LOAD_ABORT( pCurTemplate, pCurMacro, zPfxLen );
     }
-    memcpy( (void*)pzPfx, (void*)SCM_CHARS( prefix ), len );
-    pzPfx[ len ] = NUL;
-
-    /* owner */
-    pzOwner = pzPfx + len + 1;
-    len     = SCM_LENGTH( owner );
-    if (len >= 256) {
+    if (SCM_LENGTH( owner ) >= 256) {
         fprintf( stderr, zPfxMsg, zOwnLen, 256 );
         LOAD_ABORT( pCurTemplate, pCurMacro, zOwnLen );
     }
-    memcpy( (void*)pzOwner, (void*)SCM_CHARS( owner ), len );
-    pzOwner[ len ] = NUL;
 
     /*
      *  Reformat the string with the given arguments
      */
-    pzRes = asprintf( (char*)mi.pData, zScribble, pzOwner );
+    pzRes = asprintf( (char*)mi.pData, pzPrg, pzOwner );
     if (pzRes == (char*)NULL) {
-        fprintf( stderr, zAllocErr, pzProg, -1, "license string" );
+        fprintf( stderr, zAllocErr, pzPrg, -1, "license string" );
         LOAD_ABORT( pCurTemplate, pCurMacro, zFmtAlloc );
     }
 
@@ -690,6 +627,7 @@ ag_scm_license( SCM license, SCM prog_name, SCM owner, SCM prefix )
 }
 /*
  * Local Variables:
- * c-file-style: "stroustrup"
+ * c-file-style: "Stroustrup"
+ * indent-tabs-mode: nil
  * End:
  * end of expFormat.c */
