@@ -1,6 +1,6 @@
 
 /*
- *  $Id: expOutput.c,v 1.14 2001/05/09 04:24:37 bkorb Exp $
+ *  $Id: expOutput.c,v 1.19 2001/08/31 01:37:13 bkorb Exp $
  *
  *  This module implements the output file manipulation function
  */
@@ -38,11 +38,20 @@
 #  define S_IAMB      (S_ISUID|S_ISGID|S_ISVTX|S_IRWXU|S_IRWXG|S_IRWXO)
 #endif
 
-STATIC int outputDepth = 1;
+typedef struct {
+    tCC*        pzSuspendName;
+    tFpStack*   pOutDesc;
+} tSuspendName;
+
+STATIC int            suspendCt   = 0;
+STATIC int            suspAllocCt = 0;
+STATIC tSuspendName*  pSuspended  = NULL;
+STATIC int            outputDepth = 1;
 
 STATIC void addWriteAccess( char* pzFileName );
 
 
+
     EXPORT void
 removeWriteAccess( int fd )
 {
@@ -92,7 +101,7 @@ addWriteAccess( char* pzFileName )
  * doc:  
  *  Remove the current output file.  Cease processing the template for
  *  the current suffix.  It is an error if there are @code{push}-ed
- *  output files.  Use the @code{[#_ERROR 0#]} function instead.
+ *  output files.  Use the @code{(error "0")} scheme function instead.
  *  @xref{output controls}.
 =*/
     SCM
@@ -117,47 +126,144 @@ ag_scm_out_delete( void )
  * exparg: new-name, new name for the current output file
  *
  * doc:    Rename current output file.  @xref{output controls}.
+ *         Please note: changing the name will not save a temporary
+ *         file from being deleted.  It @i{may}, however, be used on the
+ *         root output file.
 =*/
     SCM
 ag_scm_out_move( SCM new_file )
 {
-    char* pz;
+    char* pz = ag_scm2zchars( new_file, "filename" );
     int   len;
 
-    if (! gh_string_p( new_file ))
-        return SCM_UNDEFINED;
-
-    len = SCM_LENGTH( new_file );
-    pz  = AGALOC( len + 1, "move-to file name" );
-    memcpy( (void*)pz, (void*)SCM_CHARS( new_file ), len );
-    pz[len] = NUL;
-
     rename( pCurFp->pzOutName, pz );
-
-    pCurFp->pzOutName = pz;  /* memory leak */
+    if ((pCurFp->flags & FPF_STATIC_NM) == 0)
+        AGFREE( (void*)pCurFp->pzOutName );
+    AGDUPSTR( pCurFp->pzOutName, pz, "file name" );
+    pCurFp->flags &= ~FPF_STATIC_NM;
     return SCM_UNDEFINED;
 }
 
 
 /*=gfunc out_pop
  *
- * what:  close current output file
+ * what:   close current output file
+ * exparg: disp, return contents of the file, optional
  * doc:  
  *  If there has been a @code{push} on the output, then close that
  *  file and go back to the previously open file.  It is an error
  *  if there has not been a @code{push}.  @xref{output controls}.
+ *
+ *  If there is no argument, no further action is taken.  Otherwise,
+ *  the argument should be @code{#t} and the contents of the file
+ *  are returned by the function.
 =*/
     SCM
-ag_scm_out_pop( void )
+ag_scm_out_pop( SCM ret_contents )
 {
+    SCM res = SCM_UNDEFINED;
+
     if (pCurFp->pPrev == (tFpStack*)NULL) {
         fputs( "ERROR:  Cannot pop output with no output pushed\n",
                stderr );
         LOAD_ABORT( pCurTemplate, pCurMacro, "expr failed" );
     }
 
+
+    if (gh_boolean_p( ret_contents ) && SCM_NFALSEP( ret_contents )) {
+        long pos;
+        pos = ftell( pCurFp->pFile );
+        res = scm_makstr( pos, 0 );
+        rewind( pCurFp->pFile );
+        if (fread( SCM_CHARS( res ), pos, 1, pCurFp->pFile ) != 1) {
+            fprintf( stderr, "Error %d (%s) rereading output\n", errno,
+                     strerror( errno ));
+            res = SCM_UNDEFINED;
+        }
+    }
+
     outputDepth--;
     closeOutput( AG_FALSE );
+    return res;
+}
+
+
+/*=gfunc out_suspend
+ *
+ * what:   suspend current output file
+ * exparg: suspName, A name tag for reactivating
+ * doc:  
+ *  If there has been a @code{push} on the output, then set aside the
+ *  output descriptor for later reactiviation with @code{(out-resume "xxx")}.
+ *  The tag name need not reflect the name of the output file.  In fact,
+ *  the output file may be an anonymous temporary file.  You may also
+ *  change the tag every time you suspend output to a file, because the
+ *  tag names are forgotten as soon as the file has been "resumed".
+=*/
+    SCM
+ag_scm_out_suspend( SCM suspName )
+{
+    SCM res = SCM_UNDEFINED;
+    tCC* pzSuspName = ag_scm2zchars( suspName, "suspend name" );
+
+    if (pCurFp->pPrev == (tFpStack*)NULL) {
+        fputs( "ERROR:  Cannot pop output with no output pushed\n",
+               stderr );
+        LOAD_ABORT( pCurTemplate, pCurMacro, "expr failed" );
+    }
+
+    if (++suspendCt > suspAllocCt) {
+        suspAllocCt += 8;
+        if (pSuspended == NULL)
+            pSuspended = (tSuspendName*)
+                AGALOC( suspAllocCt * sizeof( tSuspendName ),
+                        "suspended file list" );
+        else
+            pSuspended = (tSuspendName*)
+                AGREALOC( (void*)pSuspended,
+                          suspAllocCt * sizeof( tSuspendName ),
+                          "augmenting suspended file list" );
+    }
+
+    pSuspended[ suspendCt-1 ].pzSuspendName = gh_scm2newstr( suspName, NULL );
+    pSuspended[ suspendCt-1 ].pOutDesc      = pCurFp;
+    pCurFp = pCurFp->pPrev;
+    outputDepth--;
+
+    return res;
+}
+
+
+/*=gfunc out_resume
+ *
+ * what:   resume current output file
+ * exparg: suspName, A name tag for reactivating
+ * doc:  
+ *  If there has been a suspended output, then make that output descriptor
+ *  current again.  That output must have been suspended with the same tag
+ *  name given to this routine as its argument.
+=*/
+    SCM
+ag_scm_out_resume( SCM suspName )
+{
+    int  ix  = 0;
+    tCC* pzName = ag_scm2zchars( suspName, "resume name" );
+
+    for (; ix < suspendCt; ix++) {
+        if (strcmp( pSuspended[ ix ].pzSuspendName, pzName ) == 0) {
+            pSuspended[ ix ].pOutDesc->pPrev = pCurFp;
+            pCurFp = pSuspended[ ix ].pOutDesc;
+            free( (void*)pSuspended[ ix ].pzSuspendName );
+            if (ix < --suspendCt)
+                pSuspended[ ix ] = pSuspended[ suspendCt ];
+            ++outputDepth;
+            return SCM_UNDEFINED;
+        }
+    }
+
+    fprintf( stderr, "ERROR: no output file was suspended as ``%s''\n",
+             pzName );
+    LOAD_ABORT( pCurTemplate, pCurMacro, "resume failed" );
     return SCM_UNDEFINED;
 }
 
@@ -180,14 +286,16 @@ ag_scm_out_push_add( SCM new_file )
     if (! gh_string_p( new_file ))
         return SCM_UNDEFINED;
 
-    pzNewFile = gh_scm2newstr( new_file, NULL );
-
     p = (tFpStack*)AGALOC( sizeof( tFpStack ), "append - out file stack" );
-    p->pPrev  = pCurFp;
-    p->pzOutName = pzNewFile;  /* memory leak */
+    p->pPrev     = pCurFp;
+    p->flags     = FPF_FREE;
+
+    pzNewFile    = gh_scm2newstr( new_file, NULL );
     addWriteAccess( pzNewFile );
+    p->pFile     = fopen( pzNewFile, "a" FOPEN_BINARY_FLAG "+" );
+
+    p->pzOutName = pzNewFile;
     outputDepth++;
-    p->pFile  = fopen( pzNewFile, "a" FOPEN_BINARY_FLAG );
     pCurFp    = p;
     return SCM_UNDEFINED;
 }
@@ -196,12 +304,15 @@ ag_scm_out_push_add( SCM new_file )
 /*=gfunc out_push_new
  *
  * what:   purge and create output file
- * exparg: file-name, name of the file to create
+ * exparg: file-name, name of the file to create, optional
  *
  * doc:
  *  Leave the current output file open, but purge and create
  *  a new file that will remain open until a @code{pop} @code{delete}
- *  or @code{switch} closes it.  @xref{output controls}.
+ *  or @code{switch} closes it.  The file name is optional and, if omitted,
+ *  the output will be sent to a temporary file that will be deleted when
+ *  it is closed.
+ *  @xref{output controls}.
 =*/
     SCM
 ag_scm_out_push_new( SCM new_file )
@@ -209,17 +320,23 @@ ag_scm_out_push_new( SCM new_file )
     tFpStack* p;
     char*  pzNewFile;
 
-    if (! gh_string_p( new_file ))
-        return SCM_UNDEFINED;
-
-    pzNewFile = gh_scm2newstr( new_file, NULL );
-
     p = (tFpStack*)AGALOC( sizeof( tFpStack ), "new - out file stack" );
-    p->pPrev  = pCurFp;
-    p->pzOutName = pzNewFile;  /* memory leak */
-    unlink( pzNewFile );
+    p->pPrev = pCurFp;
+    p->flags = FPF_FREE;
+
+    if (gh_string_p( new_file )) {
+        pzNewFile = gh_scm2newstr( new_file, NULL );
+        unlink( pzNewFile );
+        addWriteAccess( pzNewFile );
+        p->pFile  = fopen( pzNewFile, "a" FOPEN_BINARY_FLAG "+" );
+    } else {
+        AGDUPSTR( pzNewFile, ".agtmp.XXXXXX", "temp file name" );
+        p->flags |= FPF_UNLINK;
+        p->pFile  = fdopen( mkstemp( pzNewFile ), "a" FOPEN_BINARY_FLAG "+" );
+    }
+
+    p->pzOutName = pzNewFile;
     outputDepth++;
-    p->pFile  = fopen( pzNewFile, "w" FOPEN_BINARY_FLAG );
     pCurFp    = p;
     return SCM_UNDEFINED;
 }
@@ -234,7 +351,8 @@ ag_scm_out_push_new( SCM new_file )
  *  Switch output files - close current file and make the current
  *  file pointer refer to the new file.  This is equivalent to
  *  @code{out-pop} followed by @code{out-push-new}, except that
- *  you may not pop the base level output file.  @xref{output controls}.
+ *  you may not pop the base level output file, but you may
+ *  @code{switch} it.  @xref{output controls}.
 =*/
     SCM
 ag_scm_out_switch( SCM new_file )
@@ -298,16 +416,22 @@ ag_scm_out_depth( void )
 /*=gfunc out_name
  *
  * what: current output file name
- * doc:  Returns the name of the current output file.
+ * doc:  Returns the name of the current output file.  If the current file
+ *       is a temporary, unnamed file, then it will scan up the chain until
+ *       a real output file name is found.
  *       @xref{output controls}.
 =*/
     SCM
 ag_scm_out_name( void )
 {
-    return gh_str02scm( pCurFp->pzOutName );
+    tFpStack* p = pCurFp;
+    while (p->flags & FPF_UNLINK)  p = p->pPrev;
+    return gh_str02scm( p->pzOutName );
 }
+
 /*
  * Local Variables:
- * c-file-style: "stroustrup"
+ * c-file-style: "Stroustrup"
+ * indent-tabs-mode: nil
  * End:
  * end of expOutput.c */
