@@ -1,7 +1,7 @@
 
 /*
  *  expState.c
- *  $Id: expState.c,v 1.30 2001/05/05 20:13:52 bkorb Exp $
+ *  $Id: expState.c,v 1.36 2001/10/13 18:51:02 bkorb Exp $
  *  This module implements expression functions that
  *  query and get state information from AutoGen data.
  */
@@ -215,15 +215,8 @@ ag_scm_base_name( void )
     SCM
 ag_scm_count( SCM obj )
 {
-    int    ent_len;
-    char*  pzEntName;
+    int ent_len = count_entries( ag_scm2zchars( obj, "ag object" ));
 
-    if (! gh_string_p( obj ))
-        return SCM_UNDEFINED;
-
-    pzEntName = gh_scm2newstr( obj, NULL );
-    ent_len = count_entries( pzEntName );
-    free( (void*)pzEntName );
     return gh_int2scm( ent_len );
 }
 
@@ -269,18 +262,11 @@ ag_scm_exist_p( SCM obj )
 {
     ag_bool x;
     SCM     res;
-    char*   pz;
 
-    if (! gh_string_p( obj ))
-        return SCM_UNDEFINED;
-
-    pz = gh_scm2newstr( obj, NULL );
-    if (findDefEntry( pz, &x ) == NULL)
+    if (findDefEntry( ag_scm2zchars( obj, "ag object" ), &x ) == NULL)
          res = SCM_BOOL_F;
     else res = SCM_BOOL_T;
 
-    free( (void*)pz );
-
     return res;
 }
 
@@ -289,27 +275,20 @@ ag_scm_exist_p( SCM obj )
  *
  * what:   test for function
  *
- * exparg: ag-name, name of AutoGen function
+ * exparg: ag-name, name of AutoGen macro
  *
- * doc:  return SCM_BOOL_T if a specified name is an AutoGen function,
- *       otherwise return SCM_BOOL_F.
+ * doc:  return SCM_BOOL_T if a specified name is a user-defined AutoGen
+ *       macro, otherwise return SCM_BOOL_F.
 =*/
     SCM
 ag_scm_ag_function_p( SCM obj )
 {
     SCM     res;
-    char*   pz;
 
-    if (! gh_string_p( obj ))
-        return SCM_UNDEFINED;
-
-    pz = gh_scm2newstr( obj, NULL );
-    if (findTemplate( pz ) == NULL)
+    if (findTemplate( ag_scm2zchars( obj, "ag user macro" )) == NULL)
          res = SCM_BOOL_F;
     else res = SCM_BOOL_T;
 
-    free( (void*)pz );
-
     return res;
 }
 
@@ -337,15 +316,12 @@ ag_scm_ag_function_p( SCM obj )
 ag_scm_match_value_p( SCM op, SCM obj, SCM test )
 {
     if (  (! gh_procedure_p( op   ))
-       || (! gh_string_p(    obj  ))
-       || (! gh_string_p(    test )) )
+       || (! gh_string_p(    obj  )) )
         return SCM_UNDEFINED;
 
-    if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS) {
-        char* pzTest = gh_scm2newstr( test, NULL );
-        fprintf( pfTrace, "searching for `%s'", pzTest );
-        free( (void*)pzTest );
-    }
+    if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS)
+        fprintf( pfTrace, "searching for `%s'",
+                 ag_scm2zchars( test, "test value" ));
 
     return find_entry_value( op, obj, test );
 }
@@ -357,21 +333,18 @@ ag_scm_match_value_p( SCM op, SCM obj, SCM test )
  *
  * exparg: ag-name, name of AutoGen value
  *
- * doc:  get the first string value associated with the name.
+ * doc:
+ *  Get the first string value associated with the name.
+ *  It will always return either the associated string value, or
+ *  the empty string.
 =*/
     SCM
 ag_scm_get( SCM obj )
 {
     tDefEntry*  pE;
     ag_bool     x;
-    char*       pz;
 
-    if (! gh_string_p( obj ))
-        return SCM_UNDEFINED;
-
-    pz = gh_scm2newstr( obj, NULL );
-    pE = findDefEntry( pz, &x );
-    free( (void*)pz );
+    pE = findDefEntry( ag_scm2zchars( obj, "ag value" ), &x );
 
     if ((pE == (tDefEntry*)NULL) || (pE->valType != VALTYP_TEXT))
         return gh_str02scm( "" );
@@ -386,21 +359,27 @@ ag_scm_get( SCM obj )
  *
  * exparg: ag-name, name of AutoGen value
  *
- * doc:  Returns the highest index associated with an array of definitions.
+ * doc:
+ *
+ *  Returns the highest index associated with an array of definitions.
+ *  This is generally, but not necessarily, one less than the
+ *  @code{count} value.  (The indexes may be specified, rendering a
+ *  non-zero based or sparse array of values.)
+ *
+ *  This is very useful for specifying the size of a zero-based array
+ *  of values where not all values are present.  For example:
+ *
+ *  @example
+ *  tMyStruct myVals[ [+ (+ 1 (high-lim "my-val-list")) +] ];
+ *  @end example
 =*/
     SCM
 ag_scm_high_lim( SCM obj )
 {
     tDefEntry*  pE;
     ag_bool     isIndexed;
-    char*       pz;
 
-    if (! gh_string_p( obj ))
-        return SCM_UNDEFINED;
-
-    pz = gh_scm2newstr( obj, NULL );
-    pE = findDefEntry( pz, &isIndexed );
-    free( (void*)pz );
+    pE = findDefEntry( ag_scm2zchars( obj, "ag value" ), &isIndexed );
 
     /*
      *  IF we did not find the entry we are looking for
@@ -426,22 +405,19 @@ ag_scm_high_lim( SCM obj )
  *
  * exparg: ag-name, name of AutoGen value
  *
- * doc: If the named object is a group definition, then "len" is
- *     the same as "count".  Otherwise, if it is one or more text
- *     definitions, then it is the sum of their string lengths.
+ * doc:  If the named object is a group definition, then "len" is
+ *       the same as "count".  Otherwise, if it is one or more text
+ *       definitions, then it is the sum of their string lengths.
+ *       If it is a single text definition, then it is equivalent to
+ *       @code{(string-length (get "ag-name"))}.
 =*/
     SCM
 ag_scm_len( SCM obj )
 {
-    char*       pz;
     int         len;
 
-    if (! gh_string_p( obj ))
-        return SCM_UNDEFINED;
+    len = entry_length( ag_scm2zchars( obj, "ag value" ));
 
-    pz = gh_scm2newstr( obj, NULL );
-    len = entry_length( pz );
-    free( (void*)pz );
     return gh_int2scm( len );
 }
 
@@ -459,14 +435,8 @@ ag_scm_low_lim( SCM obj )
 {
     tDefEntry*  pE;
     ag_bool     x;
-    char*       pz;
 
-    if (! gh_string_p( obj ))
-        return SCM_UNDEFINED;
-
-    pz = gh_scm2newstr( obj, NULL );
-    pE = findDefEntry( pz, &x );
-    free( (void*)pz );
+    pE = findDefEntry( ag_scm2zchars( obj, "ag value" ), &x );
 
     /*
      *  IF we did not find the entry we are looking for
@@ -512,24 +482,33 @@ ag_scm_tpl_file( void )
  *
  * what:   get the template file and line number
  *
+ * exparg: msg-fmt, formatting for line message, optional
+ *
  * doc:  Returns the file and line number of the current template macro
- *       using the format, "from %s line %d".
+ *       using either the default format, "from %s line %d", or else
+ *       the format you supply.  For example, if you want only the line
+ *       number, you would supply the format "%2$d".
 =*/
     SCM
-ag_scm_tpl_file_line( void )
+ag_scm_tpl_file_line( SCM fmt )
 {
+    char    zScribble[ 1024 ];
     tSCC    zFmt[] = "from %s line %d";
+    tCC*    pzFmt = zFmt;
     char*   pz;
 
+    if (gh_string_p( fmt ))
+        pzFmt = ag_scm2zchars( fmt, "file/line format" );
+
     {
         size_t  maxlen = strlen( pCurTemplate->pzFileName )
-                       + sizeof( zFmt ) + 3 * sizeof( int );
+                       + strlen( pzFmt ) + 4 * sizeof( int );
         if (maxlen >= sizeof( zScribble ))
             pz = (char*)AGALOC( maxlen, "file-line buffer" );
         else pz = zScribble;
     }
 
-    sprintf( pz, zFmt, pCurTemplate->pzFileName, pCurMacro->lineNo );
+    sprintf( pz, pzFmt, pCurTemplate->pzFileName, pCurMacro->lineNo );
 
     {
         SCM res = gh_str02scm( pz );
@@ -544,5 +523,6 @@ ag_scm_tpl_file_line( void )
 /*
  * Local Variables:
  * c-file-style: "stroustrup"
+ * indent-tabs-mode: nil
  * End:
  * end of expState.c */
