@@ -1,7 +1,7 @@
 
 /*
  *  expString.c
- *  $Id: expString.c,v 1.27 2001/05/19 22:06:22 bkorb Exp $
+ *  $Id: expString.c,v 1.37 2001/10/13 18:51:02 bkorb Exp $
  *  This module implements expression functions that
  *  manipulate string values.
  */
@@ -60,7 +60,6 @@ makeString( tCC*    pzText,
             dtaSize++;
             if ((ch == '"') || (ch == '\\'))
                 dtaSize++;
-
         }
 
         /*
@@ -171,11 +170,15 @@ makeString( tCC*    pzText,
             break;
 
         default:
+        {
+            tSCC zFmt[] = "\\%03o";
             /*
              *  sprintf is safe here, because we already computed
              *  the amount of space we will be using.
              */
-            pzDta += sprintf( pzDta, "\\%03o", ch );
+            sprintf( pzDta, zFmt, ch );
+            pzDta += 4;
+        }
         }
 
         pzScn++;
@@ -203,39 +206,63 @@ makeString( tCC*    pzText,
  * exparg: test-string, string to look for
  * exparg: string-list, list of strings to check,, list
  *
- * doc:  Return SCM_BOOL_T if the first argument is duplicated
- *      in one of the entries in the second (list) argument.
+ * doc:  Return SCM_BOOL_T if the first argument string is found
+ *      in one of the entries in the second (list-of-strings) argument.
 =*/
     SCM
 ag_scm_in_p( SCM obj, SCM list )
 {
     int   len;
-    SCM   car, res;
+    int   lenz;
+    SCM   car;
     char* pz1;
 
     if (! gh_string_p( obj ))
         return SCM_UNDEFINED;
 
-    len = scm_ilength( list );
-    pz1 = gh_scm2newstr( obj, NULL );
-    res = SCM_BOOL_F;
+    pz1  = SCM_CHARS(  obj );
+    lenz = SCM_LENGTH( obj );
 
-    while (--len >= 0) {
-        car  = SCM_CAR( list );
-        list = SCM_CDR( list );
-        if (gh_string_p( car )) {
-            char* pz2 = gh_scm2newstr( car, NULL );
-            if (strcmp( pz1, pz2 ) == 0) {
-                res = SCM_BOOL_T;
-                free( (void*)pz2 );
-                break;
-            }
-            free( (void*)pz2 );
-        }
+    /*
+     *  If the second argument is a string somehow, then treat
+     *  this as a straight out string comparison
+     */
+    if (gh_string_p( list )) {
+        if (  (SCM_LENGTH( list ) == lenz)
+           && (strncmp( pz1, SCM_CHARS( list ), lenz ) == 0) )
+            return SCM_BOOL_T;
+        return SCM_BOOL_F;
     }
 
-    free( (void*)pz1 );
-    return res;
+    len = scm_ilength( list );
+    if (len == 0)
+        return SCM_BOOL_F;
+
+    /*
+     *  Search all the lists and sub-lists passed in
+     */
+    while (len-- > 0) {
+        car  = SCM_CAR( list );
+        list = SCM_CDR( list );
+
+        /*
+         *  This routine is listed as getting a list as the second
+         *  argument.  That means that if someone builds a list and
+         *  hands it to us, it magically becomes a nested list.
+         *  This unravels that.
+         */
+        if (! gh_string_p( car )) {
+            if (ag_scm_in_p( obj, car ) == SCM_BOOL_T)
+                return SCM_BOOL_T;
+            continue;
+        }
+
+        if (  (SCM_LENGTH( car ) == lenz)
+           && (strncmp( pz1, SCM_CHARS( car ), lenz ) == 0) )
+            return SCM_BOOL_T;
+    }
+
+    return SCM_BOOL_F;
 }
 
 
@@ -371,13 +398,11 @@ ag_scm_prefix( SCM prefix, SCM text )
     size_t   pfx_size;
     SCM      res;
 
-    if ((! gh_string_p( prefix )) || (! gh_string_p( text )))
-        return SCM_UNDEFINED;
-
-    pzPfx   = SCM_CHARS( prefix );
-    pzText  = SCM_CHARS( text );
+    pzPfx   = ag_scm2zchars( prefix, "prefix" );
+    pzDta   = \
+    pzText  = ag_scm2zchars( text, "text" );
     pfx_size = strlen( pzPfx );
-    out_size = 1;
+    out_size = pfx_size;
 
     for (;;) {
         switch (*(pzText++)) {
@@ -392,27 +417,35 @@ ag_scm_prefix( SCM prefix, SCM text )
     } exit_count:;
 
     res    = scm_makstr( (scm_sizet)out_size, 0 );
+    pzText = pzDta;
     pzDta  = SCM_CHARS( res );
-    pzText = SCM_CHARS( text );
     strcpy( pzDta, pzPfx );
-    pzDta += pfx_size;
+    pzDta    += pfx_size;
+    out_size -= pfx_size;
+    pfx_size++;
 
     for (;;) {
-        switch (*(pzDta++) = *(pzText++)) {
+        char ch = *(pzText++);
+        switch (ch) {
         case NUL:
-            goto exit_copy;
+            if (out_size != 0)
+                LOAD_ABORT( pCurTemplate, pCurMacro, "(prefix ...) failed" );
+
+            return res;
 
         case '\n':
-            strcpy( pzDta, pzPfx );
-            pzDta += pfx_size;
+            *pzDta    = ch;
+            strcpy( pzDta+1, pzPfx );
+            pzDta    += pfx_size;
+            out_size -= pfx_size;
             break;
 
         default:
+            out_size--;
+            *(pzDta++) = ch;
             break;
         }
-    } exit_copy:;
-
-    return res;
+    }
 }
 
 
@@ -443,7 +476,7 @@ ag_scm_shell( SCM cmd )
     if (! gh_string_p( cmd ))
         return SCM_UNDEFINED;
     {
-        char* pz = runShell( SCM_CHARS( cmd ));
+        char* pz = runShell( ag_scm2zchars( cmd, "command" ));
         cmd   = gh_str02scm( pz );
         AGFREE( (void*)pz );
         return cmd;
@@ -467,16 +500,14 @@ ag_scm_shellf( SCM fmt, SCM alist )
 {
     int   len = scm_ilength( alist );
     char* pz;
-
-    if (! gh_string_p( fmt ))
-        return SCM_UNDEFINED;
+    char* pzFmt = ag_scm2zchars( fmt, "format" );
 
     if (len <= 0)
         return fmt;
 
-    fmt = run_printf( SCM_CHARS( fmt ), len, alist );
-    pz = runShell( SCM_CHARS( fmt ));
-    fmt   = gh_str02scm( pz );
+    fmt = run_printf( pzFmt, len, alist );
+    pz  = runShell( ag_scm2zchars( fmt, "shell script" ));
+    fmt = gh_str02scm( pz );
     AGFREE( (void*)pz );
     return fmt;
 }
@@ -496,6 +527,10 @@ ag_scm_shellf( SCM fmt, SCM alist )
  *  Contained single quotes become tripled, with the middle quote
  *  escaped with a backslash.  Normal shells will reconstitute the
  *  original string.
+ *
+ *  @strong{NOTE}:  some shells will not correctly handle unusual
+ *  non-printing characters.  This routine works for most reasonably
+ *  conventional ASCII strings.
 =*/
     SCM
 ag_scm_raw_shell_str( SCM obj )
@@ -507,10 +542,7 @@ ag_scm_raw_shell_str( SCM obj )
     char*      pz;
     SCM        res;
 
-    if (! gh_string_p( obj ))
-        return SCM_UNDEFINED;
-
-    pzDta = SCM_CHARS( obj );
+    pzDta   = ag_scm2zchars( obj, "AG Object" );
     dtaSize = strlen( pzDta ) + 3;
     pz = pzDta-1;
     for (;;) {
@@ -555,8 +587,9 @@ ag_scm_raw_shell_str( SCM obj )
  *  shell will process into the original string, almost.  It will add the
  *  escape character @code{\\} before two special characters to accomplish
  *  this: the backslash @code{\\} and double quote @code{"}.
+ *
+ *  @strong{WARNING}:
  *@*
- *  @bold{WARNING}:
  *  This function omits the extra backslash in front of a backslash, however,
  *  if it is followed by either a backquote or a dollar sign.  It must do this
  *  because otherwise it would be impossible to protect the dollar sign or
@@ -564,6 +597,10 @@ ag_scm_raw_shell_str( SCM obj )
  *  render the strings "\\$" or "\\`".  The lesser of two evils.
  *
  *  All others characters are copied directly into the output.
+ *
+ *  @strong{NOTE}: some shells will not correctly handle unusual
+ *  non-printing characters.  This routine works for most reasonably
+ *  conventional ASCII strings.
 =*/
     SCM
 ag_scm_shell_str( SCM obj )
@@ -573,10 +610,7 @@ ag_scm_shell_str( SCM obj )
     char*      pz;
     SCM        res;
 
-    if (! gh_string_p( obj ))
-        return SCM_UNDEFINED;
-
-    pzDta = SCM_CHARS( obj );
+    pzDta = ag_scm2zchars( obj, "AG Object" );
     dtaSize = 3;
     pz = pzDta;
     for (;;) {
@@ -640,7 +674,7 @@ ag_scm_shell_str( SCM obj )
  *
  * exparg: ag-name, AutoGen value name
  *
- * doc:  Create a list of all the strings that are associated
+ * doc:  Create a scheme list of all the strings that are associated
  *       with a name.  They must all be text values or we choke.
 =*/
     SCM
@@ -652,12 +686,9 @@ ag_scm_stack( SCM obj )
         tDefEntry* pDE;
         SCM        str;
 
-    if (! gh_string_p( obj ))
-        return SCM_UNDEFINED;
-
     res = SCM_EOL;
 
-    ppDE = findEntryList( SCM_CHARS( obj ) );
+    ppDE = findEntryList( ag_scm2zchars( obj, "AG Object" ));
     if (ppDE == NULL)
         return SCM_EOL;
 
@@ -696,12 +727,9 @@ ag_scm_stack( SCM obj )
     SCM
 ag_scm_kr_string( SCM str )
 {
-    tSCC    zNewLine[] = "\\n\\\n";
+    tSCC zNewLine[] = "\\n\\\n";
 
-    if (! gh_string_p( str ))
-        return SCM_UNDEFINED;
-
-    return makeString( SCM_CHARS( str ),
+    return makeString( ag_scm2zchars( str, "string" ),
                        zNewLine, sizeof( zNewLine )-1 );
 }
 
@@ -728,12 +756,9 @@ ag_scm_kr_string( SCM str )
     SCM
 ag_scm_c_string( SCM str )
 {
-    tSCC       zNewLine[] = "\\n\"\n       \"";
+    tSCC zNewLine[] = "\\n\"\n       \"";
 
-    if (! gh_string_p( str ))
-        return SCM_UNDEFINED;
-
-    return makeString( SCM_CHARS( str ),
+    return makeString( ag_scm2zchars( str, "string" ),
                        zNewLine, sizeof( zNewLine )-1 );
 }
 
@@ -758,15 +783,11 @@ ag_scm_c_string( SCM str )
     SCM
 ag_scm_string_tr_x( SCM str, SCM from_xform, SCM to_xform )
 {
-    if (   (! gh_string_p( str ))
-        || (! gh_string_p( from_xform ))
-        || (! gh_string_p( to_xform )))
-        return SCM_UNDEFINED;
     {
         char  map[ 256 ];
         int   i = 255;
-        char* pzFrom = SCM_CHARS( from_xform );
-        char* pzTo   = SCM_CHARS(   to_xform );
+        char* pzFrom = ag_scm2zchars( from_xform, "string" );
+        char* pzTo   = ag_scm2zchars(   to_xform, "string" );
 
         do  {
             map[i] = i;
@@ -805,7 +826,7 @@ ag_scm_string_tr_x( SCM str, SCM from_xform, SCM to_xform )
             }
         } mapDone:;
 
-        pzTo = SCM_CHARS( str );
+        pzTo = ag_scm2zchars( str, "string" );
         while (*pzTo != NUL) {
             *pzTo = map[ (int)*pzTo ];
             pzTo++;
@@ -816,5 +837,6 @@ ag_scm_string_tr_x( SCM str, SCM from_xform, SCM to_xform )
 /*
  * Local Variables:
  * c-file-style: "stroustrup"
+ * indent-tabs-mode: nil
  * End:
  * end of expString.c */
