@@ -1,6 +1,6 @@
 
 /*
- *  $Id: funcCase.c,v 1.27 2001/05/28 18:12:44 bkorb Exp $
+ *  $Id: funcCase.c,v 1.35 2001/10/13 18:51:02 bkorb Exp $
  *
  *  This module implements the CASE text function.
  */
@@ -118,16 +118,10 @@ Select_Compare( char* pzText, char* pzMatch )
     SCM
 ag_scm_string_contains_p( SCM text, SCM substr )
 {
-    char* pzText;
-    char* pzSubstr;
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
     SCM   res;
 
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
-
-    pzText = SCM_CHARS( text );
-    pzSubstr = SCM_CHARS( substr );
-
     res = (strstr( pzText, pzSubstr) == (char*)NULL) ? SCM_BOOL_F : SCM_BOOL_T;
 
     if (OPT_VALUE_TRACE > TRACE_EXPRESSIONS)
@@ -148,7 +142,7 @@ ag_scm_string_contains_p( SCM text, SCM substr )
  * string: "*=="
  *
  * doc:  Test to see if a string ends with a substring.
- *       strcmp(3) returns zero at the end of the string.
+ *       strcmp(3) returns zero for comparing the string ends.
 =*/
     STATIC tSuccess
 Select_Compare_End( char* pzText, char* pzMatch )
@@ -172,11 +166,9 @@ Select_Compare_End( char* pzText, char* pzMatch )
     SCM
 ag_scm_string_ends_with_p( SCM text, SCM substr )
 {
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
-
-    return (SUCCESSFUL( Select_Compare_End( SCM_CHARS( text ),
-                                            SCM_CHARS( substr ))))
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
+    return (SUCCESSFUL( Select_Compare_End( pzText, pzSubstr )))
         ? SCM_BOOL_T : SCM_BOOL_F;
 }
 
@@ -213,11 +205,9 @@ Select_Compare_Start( char* pzText, char* pzMatch )
     SCM
 ag_scm_string_starts_with_p( SCM text, SCM substr )
 {
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
-
-    return (SUCCESSFUL( Select_Compare_Start( SCM_CHARS( text ),
-                                              SCM_CHARS( substr ))))
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
+    return (SUCCESSFUL( Select_Compare_Start( pzText, pzSubstr )))
         ? SCM_BOOL_T : SCM_BOOL_F;
 }
 
@@ -249,15 +239,10 @@ Select_Compare_Full( char* pzText, char* pzMatch )
     SCM
 ag_scm_string_equals_p( SCM text, SCM substr )
 {
-    char* pzText;
-    char* pzSubstr;
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
     SCM   res;
 
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
-
-    pzText = SCM_CHARS( text );
-    pzSubstr = SCM_CHARS( substr );
     res = (strcmp( pzText, pzSubstr) == 0) ? SCM_BOOL_T : SCM_BOOL_F;
 
     if (OPT_VALUE_TRACE > TRACE_EXPRESSIONS)
@@ -280,8 +265,8 @@ ag_scm_string_equals_p( SCM text, SCM substr )
  *
  * doc:  Test to see if a string contains an equivalent string.
  *       `equivalent' means the strings match, but without regard
- *       to character case.  In other words, it is a case insensitive
- *       compare.
+ *       to character case and certain characters are considered `equivalent'.
+ *       Viz., '-', '_' and '^' are equivalent.
 =*/
     STATIC tSuccess
 Select_Equivalent( char* pzText, char* pzMatch )
@@ -303,15 +288,15 @@ Select_Equivalent( char* pzText, char* pzMatch )
     SCM
 ag_scm_string_contains_eqv_p( SCM text, SCM substr )
 {
+    tSCC zSrch[] = "search string";
     char* pzSubstr;
     SCM   res;
 
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
-    pzSubstr = strdup( SCM_CHARS( substr ));
+    AGDUPSTR( pzSubstr, ag_scm2zchars( substr, zSrch ), zSrch );
 
     upString( pzSubstr );
-    if (SUCCESSFUL( Select_Equivalent( SCM_CHARS( text ), pzSubstr )))
+    if (SUCCESSFUL( Select_Equivalent( ag_scm2zchars( text, "sample text" ),
+                                       pzSubstr )))
          res = SCM_BOOL_T;
     else res = SCM_BOOL_F;
     AGFREE( (void*)pzSubstr );
@@ -353,11 +338,9 @@ Select_Equivalent_End( char* pzText, char* pzMatch )
     SCM
 ag_scm_string_ends_eqv_p( SCM text, SCM substr )
 {
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
-
-    return (SUCCESSFUL( Select_Equivalent_End( SCM_CHARS( text ),
-                                               SCM_CHARS( substr ))))
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
+    return (SUCCESSFUL( Select_Equivalent_End( pzText, pzSubstr )))
         ? SCM_BOOL_T : SCM_BOOL_F;
 }
 
@@ -394,11 +377,9 @@ Select_Equivalent_Start( char* pzText, char* pzMatch )
     SCM
 ag_scm_string_starts_eqv_p( SCM text, SCM substr )
 {
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
-
-    return (SUCCESSFUL( Select_Equivalent_Start( SCM_CHARS( text ),
-                                                 SCM_CHARS( substr ))))
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
+    return (SUCCESSFUL( Select_Equivalent_Start( pzText, pzSubstr )))
         ? SCM_BOOL_T : SCM_BOOL_F;
 }
 
@@ -414,9 +395,14 @@ ag_scm_string_starts_eqv_p( SCM text, SCM substr )
  *
  * string: "="
  *
- * doc:  Test to see if two strings are equivalent.
- *       If the arguments are not strings, then
- *       the result of the numeric comparison is returned.
+ * doc:  Test to see if two strings are equivalent.  `equivalent' means the
+ *       strings match, but without regard to character case and certain
+ *       characters are considered `equivalent'.  Viz., '-', '_' and '^' are
+ *       equivalent.  If the arguments are not strings, then the result of the
+ *       numeric comparison is returned.
+ *
+ *       This is an overloaded operation.  If the arguments are not both
+ *       strings, then the query is passed through to @code{scm_num_eq_p()}.
 =*/
     STATIC tSuccess
 Select_Equivalent_Full( char* pzText, char* pzMatch )
@@ -431,11 +417,15 @@ ag_scm_string_eqv_p( SCM text, SCM substr )
     char* pzSubstr;
     SCM   res;
 
+    /*
+     *  We are overloading the "=" operator.  Our arguments may be
+     *  numbers...
+     */
     if (! gh_string_p( text ) || ! gh_string_p( substr ))
         return scm_num_eq_p( text, substr );
 
-    pzText = SCM_CHARS( text );
-    pzSubstr = SCM_CHARS( substr );
+    pzText   = ag_scm2zchars( text, "text to match" );
+    pzSubstr = ag_scm2zchars( substr, "match expr" );
 
     res = (streqvcmp( pzText, pzSubstr) == 0) ? SCM_BOOL_T : SCM_BOOL_F;
 
@@ -497,12 +487,10 @@ ag_scm_string_has_match_p( SCM text, SCM substr )
     SCM      res;
     regex_t  re;
 
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
+    compile_re( &re, ag_scm2zchars( substr, "match expr" ), REG_EXTENDED );
 
-    compile_re( &re, SCM_CHARS( substr ), REG_EXTENDED );
-
-    if (regexec( &re, SCM_CHARS( text ), 0, (regmatch_t*)NULL, 0 ) == 0)
+    if (regexec( &re, ag_scm2zchars( text, "text to match" ), 0,
+                 (regmatch_t*)NULL, 0 ) == 0)
          res = SCM_BOOL_T;
     else res = SCM_BOOL_F;
     regfree( &re );
@@ -513,21 +501,14 @@ ag_scm_string_has_match_p( SCM text, SCM substr )
     SCM
 ag_scm_string_has_eqv_match_p( SCM text, SCM substr )
 {
-    char* pzText;
-    char* pzSubstr;
-
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
     SCM      res;
     regex_t  re;
 
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
+    compile_re( &re, pzSubstr, REG_EXTENDED | REG_ICASE );
 
-    pzText = SCM_CHARS( text );
-    pzSubstr = SCM_CHARS( substr );
-
-    compile_re( &re, SCM_CHARS( substr ), REG_EXTENDED | REG_ICASE );
-
-    if (regexec( &re, SCM_CHARS( text ), 0, (regmatch_t*)NULL, 0 ) == 0)
+    if (regexec( &re, pzText, 0, (regmatch_t*)NULL, 0 ) == 0)
          res = SCM_BOOL_T;
     else res = SCM_BOOL_F;
     regfree( &re );
@@ -586,21 +567,13 @@ Select_Match_End( char* pzText, char* pzMatch )
     SCM
 ag_scm_string_end_match_p( SCM text, SCM substr )
 {
-    char* pzText;
-    char* pzSubstr;
-
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
     SCM        res;
     regex_t    re;
     regmatch_t m[2];
 
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
-
-    pzText = SCM_CHARS( text );
-    pzSubstr = SCM_CHARS( substr );
-
-    compile_re( &re, SCM_CHARS( substr ), REG_EXTENDED );
-    pzText = SCM_CHARS( text );
+    compile_re( &re, pzSubstr, REG_EXTENDED );
 
     if (regexec( &re, pzText, 2, m, 0 ) != 0)
          res = SCM_BOOL_F;
@@ -616,21 +589,13 @@ ag_scm_string_end_match_p( SCM text, SCM substr )
     SCM
 ag_scm_string_end_eqv_match_p( SCM text, SCM substr )
 {
-    char* pzText;
-    char* pzSubstr;
-
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
     SCM        res;
     regex_t    re;
     regmatch_t m[2];
 
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
-
-    pzText = SCM_CHARS( text );
-    pzSubstr = SCM_CHARS( substr );
-
-    compile_re( &re, SCM_CHARS( substr ), REG_EXTENDED | REG_ICASE );
-    pzText = SCM_CHARS( text );
+    compile_re( &re, pzSubstr, REG_EXTENDED | REG_ICASE );
 
     if (regexec( &re, pzText, 2, m, 0 ) != 0)
          res = SCM_BOOL_F;
@@ -694,22 +659,15 @@ Select_Match_Start( char* pzText, char* pzMatch )
     SCM
 ag_scm_string_start_match_p( SCM text, SCM substr )
 {
-    char* pzText;
-    char* pzSubstr;
-
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
     SCM        res;
     regex_t    re;
     regmatch_t m[2];
 
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
+    compile_re( &re, pzSubstr, REG_EXTENDED );
 
-    pzText = SCM_CHARS( text );
-    pzSubstr = SCM_CHARS( substr );
-
-    compile_re( &re, SCM_CHARS( substr ), REG_EXTENDED );
-
-    if (regexec( &re, SCM_CHARS( text ), 2, m, 0 ) != 0)
+    if (regexec( &re, pzText, 2, m, 0 ) != 0)
          res = SCM_BOOL_F;
     else if (m[0].rm_so != 0)
          res = SCM_BOOL_F;
@@ -723,22 +681,15 @@ ag_scm_string_start_match_p( SCM text, SCM substr )
     SCM
 ag_scm_string_start_eqv_match_p( SCM text, SCM substr )
 {
-    char* pzText;
-    char* pzSubstr;
-
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
     SCM        res;
     regex_t    re;
     regmatch_t m[2];
 
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
+    compile_re( &re, pzSubstr, REG_EXTENDED | REG_ICASE );
 
-    pzText = SCM_CHARS( text );
-    pzSubstr = SCM_CHARS( substr );
-
-    compile_re( &re, SCM_CHARS( substr ), REG_EXTENDED | REG_ICASE );
-
-    if (regexec( &re, SCM_CHARS( text ), 2, m, 0 ) != 0)
+    if (regexec( &re, pzText, 2, m, 0 ) != 0)
          res = SCM_BOOL_F;
     else if (m[0].rm_so != 0)
          res = SCM_BOOL_F;
@@ -775,7 +726,8 @@ ag_scm_string_start_eqv_match_p( SCM text, SCM substr )
  * string: "~"
  *
  * doc:  Test to see if a string fully matches a pattern.
- *       Case is not significant.
+ *       Case is not significant, but any character equivalences
+ *       must be expressed in your regular expression.
 =*/
     STATIC tSuccess
 Select_Match_Full( char* pzText, char* pzMatch )
@@ -803,21 +755,13 @@ Select_Match_Full( char* pzText, char* pzMatch )
     SCM
 ag_scm_string_match_p( SCM text, SCM substr )
 {
-    char* pzText;
-    char* pzSubstr;
-
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
     SCM        res;
     regex_t    re;
     regmatch_t m[2];
 
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
-
-    pzText = SCM_CHARS( text );
-    pzSubstr = SCM_CHARS( substr );
-
-    compile_re( &re, SCM_CHARS( substr ), REG_EXTENDED );
-    pzText = SCM_CHARS( text );
+    compile_re( &re, pzSubstr, REG_EXTENDED );
 
     if (regexec( &re, pzText, 2, m, 0 ) != 0)
          res = SCM_BOOL_F;
@@ -834,19 +778,12 @@ ag_scm_string_match_p( SCM text, SCM substr )
     SCM
 ag_scm_string_eqv_match_p( SCM text, SCM substr )
 {
-    char* pzText;
-    char* pzSubstr;
-
+    char* pzText   = ag_scm2zchars( text, "text to match" );
+    char* pzSubstr = ag_scm2zchars( substr, "match expr" );
     SCM        res;
     regex_t    re;
     regmatch_t m[2];
 
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
-        return SCM_UNDEFINED;
-
-    pzText = SCM_CHARS( text );
-    pzSubstr = SCM_CHARS( substr );
-
     compile_re( &re, pzSubstr, REG_EXTENDED | REG_ICASE );
 
     if (regexec( &re, pzText, 2, m, 0 ) != 0)
@@ -886,15 +823,15 @@ Select_Match_Always( char* pzText, char* pzMatch )
  *  (@pxref{EXPR})  The scope of the macro is up to the matching ESAC
  *  function.  Within the scope of a CASE, this string is matched against
  *  case selection macros.  There are sixteen match macros that are derived
- *  from four different ways the test may be performed.
- *  The code for each selection expression is formed as follows:
+ *  from four different ways the test may be performed, plus an "always
+ *  true" match.  The code for each selection expression is formed as follows:
  *
  *  @enumerate
  *  @item
- *  Must the match start at the beginning of the string?
+ *  Must the match start matching from the beginning of the string?
  *  If not, then the match macro code starts with an asterisk (@code{*}).
  *  @item
- *  Must the match finish at the end of the string?
+ *  Must the match finish matching at the end of the string?
  *  If not, then the match macro code ends with an asterisk (@code{*}).
  *  @item
  *  Is the match a pattern match or a string comparison?
@@ -903,6 +840,9 @@ Select_Match_Always( char* pzText, char* pzMatch )
  *  @item
  *  Is the match case sensitive?
  *  If alphabetic case is important, double the tilde or equal sign.
+ *  @item
+ *  Do you need a default match when none of the others match?
+ *  Use a single asterisk (@code{*}).
  *  @end enumerate
  *
  *  @noindent
@@ -917,10 +857,9 @@ Select_Match_Always( char* pzText, char* pzMatch )
  *  [+ ESAC +]
  *  @end example
  *
- *  @code{<full-expression>} may be any expression described in the
- *  @code{EXPR} expression function, including the use of apply-codes
- *  and value-names.  If the expression yields a number, it is
- *  converted to a decimal string.
+ *  @code{<full-expression>} (@pxref{expression syntax}) may be any expression,
+ *  including the use of apply-codes and value-names.  If the expression yields
+ *  a number, it is converted to a decimal string.
  *
  *  These case selection codes have also been implemented as
  *  Scheme expression functions using the same codes
@@ -929,13 +868,13 @@ Select_Match_Always( char* pzText, char* pzMatch )
 /*=macfunc ESAC
  *
  *  what:   Terminate the @code{CASE} Template Block
- *  situational:
  *
  *  desc:
  *    This macro ends the @code{CASE} function template block.
  *    For a complete description, @xref{CASE}.
 =*/
-MAKE_HANDLER_PROC( Case )
+    tMacro*
+mFunc_Case( tTemplate* pT, tMacro* pMac )
 {
     typedef tSuccess (t_match_proc)( char*, char* );
     /*
@@ -1069,7 +1008,8 @@ static tpLoadProc apSelectOnly[ FUNC_CT ] = { (tpLoadProc)NULL };
  *  This function is called to set up (load) the macro
  *  when the template is first read in (before processing).
  */
-MAKE_LOAD_PROC( Case )
+    tMacro*
+mLoad_Case( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     size_t         srcLen     = (size_t)pMac->res;   /* macro len  */
     tCaseStack     save_stack = current_case;
@@ -1171,14 +1111,18 @@ MAKE_LOAD_PROC( Case )
  *  alias: ~
  *  alias: =
  *  alias: *
- *  situational:
  *
  *  desc:
  *    This macro selects a block of text by matching an expression
  *    against the sample text expression evaluated in the @code{CASE}
  *    macro.  @xref{CASE}.
+ *
+ *    You do not specify a @code{SELECT} macro with the word ``select''.
+ *    Instead, you must use one of the 17 match operators described in
+ *    the @code{CASE} macro description.
 =*/
-MAKE_LOAD_PROC( Select )
+    tMacro*
+mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     const char*    pzScan = *ppzScan;  /* text after macro */
     char*          pzCopy = pT->pNext; /* next text dest   */
@@ -1306,5 +1250,6 @@ MAKE_LOAD_PROC( Select )
 /*
  * Local Variables:
  * c-file-style: "stroustrup"
+ * indent-tabs-mode: nil
  * End:
  * end of funcCase.c */
