@@ -1,6 +1,6 @@
 
 /*
- *  $Id: funcEval.c,v 1.34 2001/05/05 20:13:53 bkorb Exp $
+ *  $Id: funcEval.c,v 1.38 2001/08/26 03:37:17 bkorb Exp $
  *
  *  This module evaluates macro expressions.
  */
@@ -42,7 +42,7 @@ STATIC int exprType( char* pz );
 
 
     EXPORT char*
-resolveSCM( SCM s, ag_bool*  pMustFree )
+resolveSCM( SCM s )
 {
     static char z[32];
     char*  pzRes = z;
@@ -50,18 +50,12 @@ resolveSCM( SCM s, ag_bool*  pMustFree )
 
     switch (gh_type_e( s )) {
     case GH_TYPE_BOOLEAN:         
-        z[0] = SCM_NFALSEP(s) ? '1' : '0'; z[1] = NUL; break;
+        z[0] = SCM_NFALSEP(s) ? '1' : '0'; z[1] = NUL;
+        break;
 
     case GH_TYPE_STRING:
     case GH_TYPE_SYMBOL:
-        len = SCM_LENGTH(s);
-        if (len >= sizeof( z )) {
-            pzRes = AGALOC( len + 1, "SCM result string" );
-            *pMustFree = AG_TRUE;
-        }
-
-        memcpy( pzRes, SCM_CHARS(s), len );
-        pzRes[ len ] = NUL;
+        pzRes = ag_scm2zchars( s, "SCM Result" );
         break;
 
     case GH_TYPE_CHAR:
@@ -74,7 +68,7 @@ resolveSCM( SCM s, ag_bool*  pMustFree )
         pzRes = "** Pair **"; break;
 
     case GH_TYPE_NUMBER:
-        snprintf( z, sizeof(z), "%ld", gh_scm2long(s) ); break;
+        snprintf( z, sizeof(z), "%d", gh_scm2ulong(s) ); break;
 
     case GH_TYPE_PROCEDURE:
 #ifdef SCM_SUBR_ENTRY
@@ -256,7 +250,7 @@ evalExpression( ag_bool* pMustFree )
             *pMustFree = AG_FALSE;
         }
 
-        pzText = resolveSCM( res, pMustFree );
+        pzText = resolveSCM( res );
         break;
     }
 
@@ -380,12 +374,14 @@ eval( const char* pzExpr )
  *   macro is inferred.  The result of the expression evaluation
  *   (@pxref{expression syntax}) is written to the current output.
 =*/
-MAKE_HANDLER_PROC( Expr )
+    tMacro*
+mFunc_Expr( tTemplate* pT, tMacro* pMac )
 {
     ag_bool needFree;
     char* pz = evalExpression( &needFree );
 
     fputs( pz, pCurFp->pFile );
+    fflush( pCurFp->pFile );
     if (needFree)
         AGFREE( (void*)pz );
 
@@ -420,7 +416,8 @@ exprType( char* pz )
 /*
  *  mLoad_Expression
  */
-MAKE_LOAD_PROC( Expr )
+    tMacro*
+mLoad_Expr( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     char*    pzCopy; /* next text dest   */
     tCC*     pzSrc  = (const char*)pMac->ozText; /* macro text */
