@@ -1,6 +1,6 @@
 
 /*
- *  $Id: funcFor.c,v 1.26 2001/05/28 18:15:38 bkorb Exp $
+ *  $Id: funcFor.c,v 1.31 2001/10/13 18:51:02 bkorb Exp $
  *
  *  This module implements the FOR text function.
  */
@@ -170,7 +170,9 @@ ag_scm_for_index( SCM which )
  * exparg: from, the initial index for the AutoGen FOR macro
  *
  * doc:  This function records the initial index information
- *       for an AutoGen FOR function.  @xref{FOR}.
+ *       for an AutoGen FOR function.
+ *       Outside of the FOR macro itself, this function will emit an error.
+ *       @xref{FOR}.
 =*/
     SCM
 ag_scm_for_from( SCM from )
@@ -188,7 +190,9 @@ ag_scm_for_from( SCM from )
  * exparg: to, the final index for the AutoGen FOR macro
  *
  * doc:  This function records the terminating value information
- *       for an AutoGen FOR function.  @xref{FOR}.
+ *       for an AutoGen FOR function.
+ *       Outside of the FOR macro itself, this function will emit an error.
+ *       @xref{FOR}.
 =*/
     SCM
 ag_scm_for_to( SCM to )
@@ -205,9 +209,10 @@ ag_scm_for_to( SCM to )
  * what:   set iteration step
  * exparg: by, the iteration increment for the AutoGen FOR macro
  *
- * doc:
- *    This function records the "step by" information
- *    for an AutoGen FOR function.  @xref{FOR}.
+ * doc:  This function records the "step by" information
+ *       for an AutoGen FOR function.
+ *       Outside of the FOR macro itself, this function will emit an error.
+ *       @xref{FOR}.
 =*/
     SCM
 ag_scm_for_by( SCM by )
@@ -225,17 +230,17 @@ ag_scm_for_by( SCM by )
  * exparg: separator, the text to insert between the output of
  *         each FOR iteration
  *
- * doc:
- *  This function records the separation string that is to be inserted
- *  between each iteration of an AutoGen FOR function.  This is often
- *  nothing more than a comma.  @xref{FOR}.
+ * doc:  This function records the separation string that is to be inserted
+ *       between each iteration of an AutoGen FOR function.  This is often
+ *       nothing more than a comma.
+ *       Outside of the FOR macro itself, this function will emit an error.
 =*/
     SCM
 ag_scm_for_sep( SCM obj )
 {
-    if ((! pFS->for_loading) || (! gh_string_p( obj )))
+    if (! pFS->for_loading)
         return SCM_UNDEFINED;
-    pFS->for_pzSep = strdup( SCM_CHARS( obj ));
+    pFS->for_pzSep = strdup( ag_scm2zchars( obj, "separator str" ));
     return SCM_BOOL_T;
 }
 
@@ -416,6 +421,7 @@ doForByStep( tTemplate* pT,
             break;
 
         fputs( pFS->for_pzSep, pCurFp->pFile );
+        fflush( pCurFp->pFile );
         pFS->for_firstFor = AG_FALSE;
         pFS->for_index = nextIdx;
     }
@@ -478,6 +484,7 @@ doForEach( tTemplate*   pT,
          *  Emit the iteration separation
          */
         fputs( pFS->for_pzSep, pCurFp->pFile );
+        fflush( pCurFp->pFile );
     }
 
     currDefCtx = stack;  /* Restore the def context */
@@ -506,8 +513,8 @@ doForEach( tTemplate*   pT,
  *
  *  The first argument must be the name of an AutoGen value.  If there is
  *  no value associated with the name, the FOR loop block is skipped
- *  entirely.  The scope of the @code{FOR} function extends to the ENDFOR
- *  macro that contains this name.
+ *  entirely.  The scope of the @code{FOR} function extends to the
+ *  corresponding ENDFOR macro.
  *
  *  If there are any further arguments, if the first character is either
  *  a semi-colon (@code{;}) or an opening parenthesis (@code{(}), then
@@ -557,13 +564,13 @@ doForEach( tTemplate*   pT,
 /*=macfunc ENDFOR
  *
  *  what:   Terminates the @code{FOR} function template block
- *  situational:
  *
  *  desc:
  *    This macro ends the @code{FOR} function template block.
  *    For a complete description @xref{FOR}.
 =*/
-MAKE_HANDLER_PROC( For )
+    tMacro*
+mFunc_For( tTemplate* pT, tMacro* pMac )
 {
     tMacro*     pMRet = pT->aMacros + pMac->endIndex;
     ag_bool     isIndexed;
@@ -635,7 +642,8 @@ MAKE_HANDLER_PROC( For )
 
 tSCC zNoEnd[] = "%s ERROR:  FOR loop `%s' does not end\n";
 
-MAKE_LOAD_PROC( For )
+    tMacro*
+mLoad_For( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     char*        pzCopy = pT->pNext; /* next text dest   */
     const char*  pzSrc  = (const char*)pMac->ozText; /* macro text */
@@ -712,5 +720,6 @@ MAKE_LOAD_PROC( For )
 /*
  * Local Variables:
  * c-file-style: "stroustrup"
+ * indent-tabs-mode: nil
  * End:
  * end of funcFor.c */
