@@ -1,6 +1,6 @@
 
 /*
- *  $Id: funcIf.c,v 1.16 2001/05/28 18:15:01 bkorb Exp $
+ *  $Id: funcIf.c,v 1.19 2001/10/13 18:51:02 bkorb Exp $
  *
  *  This module implements the _IF text function.
  */
@@ -123,13 +123,13 @@ eval_true( void )
 /*=macfunc ENDIF
  *
  *  what:   Terminate the @code{IF} Template Block
- *  situational:
  *
  *  desc:
  *    This macro ends the @code{IF} function template block.
  *    For a complete description @xref{IF}.
 =*/
-MAKE_HANDLER_PROC( If )
+    tMacro*
+mFunc_If( tTemplate* pT, tMacro* pMac )
 {
     tMacro* pRet = pT->aMacros + pMac->endIndex;
     tMacro* pIf  = pMac;
@@ -206,13 +206,13 @@ MAKE_HANDLER_PROC( If )
 /*=macfunc ENDWHILE
  *
  *  what:   Terminate the @code{WHILE} Template Block
- *  situational:
  *
  *  desc:
  *    This macro ends the @code{WHILE} function template block.
  *    For a complete description @xref{WHILE}.
 =*/
-MAKE_HANDLER_PROC( While )
+    tMacro*
+mFunc_While( tTemplate* pT, tMacro* pMac )
 {
     tMacro* pRet = pT->aMacros + pMac->endIndex;
     int     ct   = 0;
@@ -257,7 +257,6 @@ STATIC tLoadProc mLoad_Elif, mLoad_Else;
 /*=macfunc ELIF
  *
  *  what:   Alternate Conditional Template Block
- *  situational:
  *
  *  desc:
  *    This macro must only appear after an @code{IF} function, and
@@ -266,7 +265,8 @@ STATIC tLoadProc mLoad_Elif, mLoad_Else;
  *    @code{IF} function.  Its expression argument is evaluated as are
  *    the arguments to @code{IF}.  For a complete description @xref{IF}.
 =*/
-MAKE_LOAD_PROC( Elif )
+    tMacro*
+mLoad_Elif( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     /*
      *  Load the expression
@@ -282,7 +282,6 @@ MAKE_LOAD_PROC( Elif )
 /*=macfunc ELSE
  *
  *  what:   Alternate Template Block
- *  situational:
  *
  *  desc:
  *    This macro must only appear after an @code{IF} function,
@@ -290,7 +289,8 @@ MAKE_LOAD_PROC( Elif )
  *    It denotes the start of an alternate template block for
  *    the @code{IF} function.  For a complete description @xref{IF}.
 =*/
-MAKE_LOAD_PROC( Else )
+    tMacro*
+mLoad_Else( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     /*
      *  After processing an "ELSE" macro,
@@ -313,14 +313,16 @@ MAKE_LOAD_PROC( Else )
 }
 
 
-MAKE_LOAD_PROC( Ending )
+    tMacro*
+mLoad_Ending( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     memset( (void*)pMac, 0, sizeof( *pMac ));
     return (tMacro*)NULL;
 }
 
 
-MAKE_LOAD_PROC( If )
+    tMacro*
+mLoad_If( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     size_t         srcLen = (size_t)pMac->res;         /* macro len  */
 
@@ -387,7 +389,8 @@ MAKE_LOAD_PROC( If )
 }
 
 
-MAKE_LOAD_PROC( While )
+    tMacro*
+mLoad_While( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     size_t         srcLen = (size_t)pMac->res;         /* macro len  */
 
@@ -446,19 +449,38 @@ MAKE_LOAD_PROC( While )
  *
  * what:   Make the output file be writable
  *
- * doc:    This function will set the current output file to be writable.
- *         This is only effective if the command line option
- *         @code{--not-writable} has not been specified.  This state
+ * exparg: + set? + boolean arg, false to make output non-writable + opt
+ *
+ * doc:    This function will set the current output file to be writable
+ *         (or not).  This is only effective if neither the @code{--writable}
+ *         nor @code{--not-writable} have been specified.  This state
  *         is reset when the current suffix's output is complete.
 =*/
     SCM
-ag_scm_set_writable( void )
+ag_scm_set_writable( SCM set )
 {
-    SET_OPT_WRITABLE;
+    tSCC zWarn[] =
+        "Warning: (set-writable) function in %s on line %d:\n"
+        "\toverridden by invocation option\n";
+
+    switch (STATE_OPT( WRITABLE )) {
+    case OPTST_DEFINED:
+    case OPTST_PRESET:
+        fprintf( stderr, zWarn, pCurTemplate->pzFileName, pCurMacro->lineNo );
+        break;
+
+    default:
+        if (gh_boolean_p( set ) && (set == SCM_BOOL_F))
+            CLEAR_OPT( WRITABLE );
+        else
+            SET_OPT_WRITABLE;
+    }
+
     return SCM_UNDEFINED;
 }
 /*
  * Local Variables:
  * c-file-style: "stroustrup"
+ * indent-tabs-mode: nil
  * End:
  * end of funcIf.c */
