@@ -1,6 +1,6 @@
 
 /*
- *  $Id: functions.c,v 1.20 2001/05/05 20:13:53 bkorb Exp $
+ *  $Id: functions.c,v 1.24 2001/10/13 18:51:02 bkorb Exp $
  *
  *  This module implements text functions.
  */
@@ -32,6 +32,7 @@
 #include "autogen.h"
 
 tSCC zCantInc[] = "cannot include file";
+tSCC zTrcFmt[] = "%-10s (%2X) in %s at line %d\n";
 
 
 /*=macfunc INCLUDE
@@ -51,7 +52,8 @@ tSCC zCantInc[] = "cannot include file";
  *  @samp{CASE} and extending through the respective terminating
  *  macro functions.
 =*/
-MAKE_HANDLER_PROC( Include )
+    tMacro*
+mFunc_Include( tTemplate* pT, tMacro* pMac )
 {
     tTemplate* pNewTpl;
     ag_bool    needFree;
@@ -108,30 +110,39 @@ MAKE_HANDLER_PROC( Include )
  *  unnamed:
  *
  *  desc:
- *  The macro text has started with a name not known to AutoGen.
- *  If, at template instantiation time, it turns out to be the
- *  name of a template block, then the macro invokes that template.
- *  If it is not, then it is a conditional expression that is
- *  evaluated only if the name is defined at the time the macro
- *  is invoked.
  *
- *  You may not specify @code{UNKNOWN} specifically.
+ *  The macro text has started with a name not known to AutoGen.  If, at run
+ *  time, it turns out to be the name of a defined macro, then that macro is
+ *  invoked.  If it is not, then it is a conditional expression that is
+ *  evaluated only if the name is defined at the time the macro is invoked.
+ *
+ *  You may not specify @code{UNKNOWN} explicitly.
 =*/
-MAKE_HANDLER_PROC( Unknown )
+    tMacro*
+mFunc_Unknown( tTemplate* pT, tMacro* pMac )
 {
     tTemplate* pInv = findTemplate( pT->pzTemplText + pMac->ozName );
     if (pInv != (tTemplate*)NULL) {
+        if (OPT_VALUE_TRACE >= TRACE_EVERYTHING)
+            fprintf( pfTrace, zTrcFmt, "remapped to Define", pMac->funcCode,
+                     pT->pzFileName, pMac->lineNo );
         pMac->funcCode    = FTYP_DEFINE;
         pMac->funcPrivate = (void*)pInv;
         parseMacroArgs( pT, pMac );
         return mFunc_Define( pT, pMac );
     }
 
+    if (OPT_VALUE_TRACE >= TRACE_EVERYTHING) {
+        fprintf( pfTrace, zTrcFmt, "remapped to Expr", pMac->funcCode,
+                 pT->pzFileName, pMac->lineNo );
+        fprintf( pfTrace, "\tbased on %s\n", pT->pzTemplText + pMac->ozName );
+    }
+
     pMac->funcCode = FTYP_EXPR;
-    if (pMac->ozText == 0)
+    if (pMac->ozText == 0) {
         pMac->res = EMIT_VALUE;
 
-    else {
+    } else {
         char* pzExpr = pT->pzTemplText + pMac->ozText;
         switch (*pzExpr) {
         case ';':
@@ -152,6 +163,9 @@ MAKE_HANDLER_PROC( Unknown )
         default:
             pMac->res = EMIT_STRING;
         }
+
+        if (OPT_VALUE_TRACE >= TRACE_EVERYTHING)
+            fprintf( pfTrace, "\tcode %X -- %s\n", pMac->res, pzExpr );
     }
 
     return mFunc_Expr( pT, pMac );
@@ -165,7 +179,8 @@ MAKE_HANDLER_PROC( Unknown )
  *  unnamed:
  *  load_proc:
 =*/
-MAKE_HANDLER_PROC( Bogus )
+    tMacro*
+mFunc_Bogus( tTemplate* pT, tMacro* pMac )
 {
     tSCC z[] = "%d (%s) is an unknown macro function, or has no handler";
     char* pz = asprintf( z, pMac->funcCode, (pMac->funcCode < FUNC_CT)
@@ -176,14 +191,14 @@ MAKE_HANDLER_PROC( Bogus )
 }
 
 
-
 /*=macfunc TEXT
  *
  *  what:  A block of text to be emitted.
  *  handler_proc:
  *  unnamed:
 =*/
-MAKE_HANDLER_PROC( Text )
+    tMacro*
+mFunc_Text( tTemplate* pT, tMacro* pMac )
 {
     fputs( pT->pzTemplText + pMac->ozText, pCurFp->pFile );
     fflush( pCurFp->pFile );
@@ -198,16 +213,16 @@ MAKE_HANDLER_PROC( Text )
  *  alias:  #
  *  load_proc:
  *
- *  situational:
  *    This function can be specified by the user, but there will
  *    never be a situation where it will be invoked at emit time.
  *    The macro is actually removed from the internal representation.
  *
  *  desc:
- *    If the specified function code is @code{#}, then the
+ *    If the native macro name code is @code{#}, then the
  *    entire macro function is treated as a comment and ignored.
 =*/
-MAKE_LOAD_PROC( Comment )
+    tMacro*
+mLoad_Comment( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     memset( (void*)pMac, 0, sizeof( *pMac ));
     return pMac;
@@ -220,7 +235,8 @@ MAKE_LOAD_PROC( Comment )
  *  Move any text into the text offset field.
  *  This is used as the default load mechanism.
  */
-MAKE_LOAD_PROC( Unknown )
+    tMacro*
+mLoad_Unknown( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     char*          pzCopy = pT->pNext;
     const char*    pzSrc;
@@ -319,7 +335,8 @@ MAKE_LOAD_PROC( Unknown )
  *  However, the load function pointer for those functions points
  *  here, until an "IF" function is encountered.
  */
-MAKE_LOAD_PROC( Bogus )
+    tMacro*
+mLoad_Bogus( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     const char*    pzSrc  = (const char*)pMac->ozText; /* macro text */
     long           srcLen = (long)pMac->res;           /* macro len  */
@@ -344,6 +361,7 @@ MAKE_LOAD_PROC( Bogus )
 }
 /*
  * Local Variables:
- * c-file-style: "stroustrup"
+ * c-file-style: "Stroustrup"
+ * indent-tabs-mode: nil
  * End:
  * end of functions.c */
