@@ -1,5 +1,5 @@
 /*
- * AutoGen Globally exported prototypes Tue Jun  5 20:50:30 PDT 2001
+ * AutoGen Globally exported prototypes Sat Oct 27 11:49:54 PDT 2001
  */
 #ifndef AUTOGEN_PROTOTYPES_H
 #define AUTOGEN_PROTOTYPES_H
@@ -10,12 +10,12 @@ void
 closeServer( void );
 int
 chainOpen( int       stdinFd,
-           tpChar*   ppArgs,
+           tCC**     ppArgs,
            pid_t*    pChild  );
 pid_t
-openServer( tFdPair* pPair, tpChar* ppArgs );
+openServer( tFdPair* pPair, tCC** ppArgs );
 pid_t
-openServerFP( tpfPair* pfPair, tpChar* ppArgs );
+openServerFP( tpfPair* pfPair, tCC** ppArgs );
 char*
 loadData( FILE* fp );
 char*
@@ -25,16 +25,16 @@ runShell( const char*  pzCmd );
 
 void
 doOptions( int arg_ct, char** arg_vec );
-const char*
-getDefine( const char* pzDefName );
+tCC*
+getDefine( tCC* pzDefName );
 unsigned int
-doEscapeChar( const char* pzIn, char* pRes );
+doEscapeChar( tCC* pzIn, char* pRes );
 char*
 spanQuote( char* pzQte );
-const char*
-skipScheme( const char* pzSrc,  const char* pzEnd );
-const char*
-skipExpression( const char* pzSrc, size_t len );
+tCC*
+skipScheme( tCC* pzSrc,  tCC* pzEnd );
+tCC*
+skipExpression( tCC* pzSrc, size_t len );
 
 /* exports from defDirect.c */
 
@@ -109,7 +109,7 @@ parseMacroArgs( tTemplate* pT, tMacro* pMac );
 /* exports from funcEval.c */
 
 char*
-resolveSCM( SCM s, ag_bool*  pMustFree );
+resolveSCM( SCM s );
 char*
 evalExpression( ag_bool* pMustFree );
 SCM
