@@ -1,6 +1,6 @@
 
 /*
- *  autoopts.h  $Id: autoopts.h,v 2.19 2001/05/19 22:06:22 bkorb Exp $
+ *  autoopts.h  $Id: autoopts.h,v 2.21 2001/09/23 01:31:14 bkorb Exp $
  *
  *  This file defines all the global structures and special values
  *  used in the automated option processing library.
@@ -130,6 +130,8 @@
 #define NAMED_OPTS(po) \
         (((po)->fOptSet & (OPTPROC_SHORTOPT | OPTPROC_LONGOPT)) == 0)
 
+#define SKIP_OPT(p)  (((p)->fOptState & (OPTST_DOCUMENT|OPTST_OMITTED)) != 0)
+
 typedef int tSuccess;
 #define SUCCESS         ((tSuccess)0)
 #define FAILURE         ((tSuccess)-1)
@@ -176,6 +178,60 @@ typedef struct {
     char*      pzOptArg;
 } tOptState;
 
+#if defined( __STDC__ ) || defined( __cplusplus )
+#  ifndef PROTO
+#    define PROTO(s) s
+#  endif
+
+/*
+ *  All this "DEF_PROC_*" weirdness is so that I can compile the option
+ *  processing code with a K&R compiler.  Yuck.  Don't use them unless
+ *  you really have to.  (Still better than ugly if-defs, tho.)
+ */
+#define DEF_PROC_0( id ) \
+          id( void ); \
+          id( void )
+
+#define DEF_PROC_1( id, a1_type, a1_name ) \
+          id( a1_type a1_name ); \
+          id( a1_type a1_name )
+
+#define DEF_PROC_2( id, a1t, a1n, a2t, a2n ) \
+          id( a1t a1n, a2t a2n ); \
+          id( a1t a1n, a2t a2n )
+
+#define DEF_PROC_3( id, a1t, a1n, a2t, a2n, a3t, a3n ) \
+          id( a1t a1n, a2t a2n, a3t a3n ); \
+          id( a1t a1n, a2t a2n, a3t a3n )
+
+#define DEF_PROC_4( id, a1t, a1n, a2t, a2n, a3t, a3n, a4t, a4n ) \
+          id( a1t a1n, a2t a2n, a3t a3n, a4t a4n ); \
+          id( a1t a1n, a2t a2n, a3t a3n, a4t a4n )
+
+#else
+#  ifndef PROTO
+#    define PROTO(s) ()
+#  endif
+
+#define DEF_PROC_0( id ) \
+          id()
+
+#define DEF_PROC_1( id, a1_type, a1_name ) \
+          id( a1_name ) a1_type a1_name;
+
+#define DEF_PROC_2( id, a1t, a1n, a2t, a2n ) \
+          id( a1n, a2n ) a1t a1n; a2t a2n;
+
+#define DEF_PROC_3( id, a1t, a1n, a2t, a2n, a3t, a3n ) \
+          id( a1n, a2n, a3n ) \
+          a1t a1n; a2t a2n; a3t a3n;
+
+#define DEF_PROC_4( id, a1t, a1n, a2t, a2n, a3t, a3n, a4t, a4n ) \
+          id( a1n, a2n, a3n ) \
+          a1t a1n; a2t a2n; a3t a3n; a4t a4n;
+
+#endif
+
 #ifdef MEMDEBUG
    extern void* ag_alloc( size_t, const char* );
    extern void* ag_realloc( void*, size_t, const char* );
