@@ -1,6 +1,6 @@
 
 /*
- *  options.h  $Id: options.h,v 2.28 2001/06/06 03:08:37 bkorb Exp $
+ *  options.h  $Id: options.h,v 2.31 2001/09/23 01:31:14 bkorb Exp $
  *
  *  This file defines all the global structures and special values
  *  used in the automated option processing library.
@@ -84,6 +84,7 @@
 #define OPTST_DOCUMENT     0x08000  /* opt is for documentation only     */
 #define OPTST_IMM          0x10000  /* process option on first pass      */
 #define OPTST_DISABLE_IMM  0x20000  /* process disablement on first pass */
+#define OPTST_OMITTED      0x40000  /* compiled out of program           */
 
 #define OPTST_PERSISTENT   0xFFF00  /* mask of flags that do not change  */
 
@@ -237,7 +238,7 @@ struct specOptIndex {
  *  fields.  This way, the "optionProcess()" routine may exit with an
  *  informative message instead of, for example, page faulting.
  */
-#define  OPTIONS_STRUCT_VERSION  37250
+#define  OPTIONS_STRUCT_VERSION  45058
 
 struct options {
     const int         structVersion;
@@ -275,53 +276,11 @@ extern "C" {
 #    define PROTO(s) s
 #  endif
 
-/*
- *  All this "DEF_PROC_*" weirdness is so that I can compile the option
- *  processing code with a K&R compiler.  Yuck.  Don't use them unless
- *  you really have to.  (Still better than ugly if-defs, tho.)
- */
-#define DEF_PROC_0( id ) \
-          id( void ); \
-          id( void )
-
-#define DEF_PROC_1( id, a1_type, a1_name ) \
-          id( a1_type a1_name ); \
-          id( a1_type a1_name )
-
-#define DEF_PROC_2( id, a1t, a1n, a2t, a2n ) \
-          id( a1t a1n, a2t a2n ); \
-          id( a1t a1n, a2t a2n )
-
-#define DEF_PROC_3( id, a1t, a1n, a2t, a2n, a3t, a3n ) \
-          id( a1t a1n, a2t a2n, a3t a3n ); \
-          id( a1t a1n, a2t a2n, a3t a3n )
-
-#define DEF_PROC_4( id, a1t, a1n, a2t, a2n, a3t, a3n, a4t, a4n ) \
-          id( a1t a1n, a2t a2n, a3t a3n, a4t a4n ); \
-          id( a1t a1n, a2t a2n, a3t a3n, a4t a4n )
-
 #else
 #  ifndef PROTO
 #    define PROTO(s) ()
 #  endif
 
-#define DEF_PROC_0( id ) \
-          id()
-
-#define DEF_PROC_1( id, a1_type, a1_name ) \
-          id( a1_name ) a1_type a1_name;
-
-#define DEF_PROC_2( id, a1t, a1n, a2t, a2n ) \
-          id( a1n, a2n ) a1t a1n; a2t a2n;
-
-#define DEF_PROC_3( id, a1t, a1n, a2t, a2n, a3t, a3n ) \
-          id( a1n, a2n, a3n ) \
-          a1t a1n; a2t a2n; a3t a3n;
-
-#define DEF_PROC_4( id, a1t, a1n, a2t, a2n, a3t, a3n, a4t, a4n ) \
-          id( a1n, a2n, a3n ) \
-          a1t a1n; a2t a2n; a3t a3n; a4t a4n;
-
 #  ifndef void
 #    define void int
 #  endif
