@@ -1,20 +1,21 @@
 /*  -*- Mode: C -*-
  *
- *  $Id: getdefs.h,v 2.5 2000/09/18 02:04:05 bkorb Exp $
+ *  $Id: getdefs.h,v 2.7 2001/08/23 03:23:50 bkorb Exp $
  *
  *    getdefs copyright 1999 Bruce Korb
  *
  *  Author:            Bruce Korb <bkorb@gnu.org>
  *  Maintainer:        Bruce Korb <bkorb@gnu.org>
  *  Created:           Mon Jun 30 15:35:12 1997
- *  Last Modified:     $Date: 2000/09/18 02:04:05 $
+ *  Last Modified:     $Date: 2001/08/23 03:23:50 $
  *            by:      Bruce Korb <bkorb@gnu.org>
  */
 
 #ifndef GETDEFS_HEADER
 #define GETDEFS_HEADER
 
-#include <sys/types.h>
+#include "config.h"
+
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -27,7 +28,9 @@
 #include <unistd.h>
 #include <utime.h>
 
-#include "config.h"
+#ifndef HAVE_SNPRINTF
+#  include "snprintfv/snprintfv.h"
+#endif
 
 #ifndef HAVE_STRSIGNAL
 #  include "compat/strsignal.c"
@@ -103,10 +106,10 @@ typedef int tSuccess;
 /*
  *  Index database string pointers.
  */
-MODE char*    pzIndexText VALUE( (char*)NULL );
-MODE char*    pzEndIndex  VALUE( (char*)NULL );
-MODE char*    pzIndexEOF  VALUE( (char*)NULL );
-MODE size_t   indexAlloc  VALUE( 0 );
+MODE char*    pzIndexText VALUE( (char*)NULL ); /* all the text    */
+MODE char*    pzEndIndex  VALUE( (char*)NULL ); /* end of current  */
+MODE char*    pzIndexEOF  VALUE( (char*)NULL ); /* end of file     */
+MODE size_t   indexAlloc  VALUE( 0 );           /* allocation size */
 
 /*
  *  Name of program to process output (normally ``autogen'')
