@@ -8,7 +8,7 @@
  * Last Modified:      Thu Apr  6 02:20:12 2000
  *            by:      Gary V. Vaughan <gvv@techie.com>
  * ---------------------------------------------------------------------
- * @(#) $Id: snprintfv.c,v 1.5 2000/07/27 15:46:01 bkorb Exp $
+ * @(#) $Id: snprintfv.c,v 1.6 2001/08/11 17:53:57 bkorb Exp $
  * ---------------------------------------------------------------------
  */
 
@@ -497,57 +497,35 @@ va_list_to_argv (format, ap)
 
 	       Thanks to Robert Lipe <robertlipe@usa.net> for explaining all
 	       this to me. */
+#	    define SNV_IS(t)  ((argtypes[index] & (t)) == (t))
 	    switch(argtypes[index] & SNV_TYPE_MASK)
 	    {
 	    case SNV_INT:
-		if (argtypes[index] & (SNV_FLAG_LONG|SNV_FLAG_UNSIGNED))
+		if (SNV_IS(SNV_FLAG_LONG|SNV_FLAG_UNSIGNED))
 		{
-		    args[index] = (snv_constpointer)snv_new(unsigned long, 1);
-		    *(unsigned long*)args[index] = va_arg(ap, unsigned long);
+		    args[index] = SNV_UINT_TO_POINTER(va_arg(ap, unsigned long));
 		}
-		else if (argtypes[index] & SNV_FLAG_LONG)
+		else if (SNV_IS(SNV_FLAG_LONG))
 		{
-		    args[index] = (snv_constpointer)snv_new(long, 1);
-		    *(long*)args[index] = va_arg(ap, long);
+		    args[index] = SNV_INT_TO_POINTER(va_arg(ap, long));
 		}
-		/* The ISO C variadic argument promotion rules obsolete
-		   this section:
-		else if (argtypes[index] & (SNV_FLAG_SHORT|SNV_FLAG_UNSIGNED))
+		else if (SNV_IS(SNV_FLAG_UNSIGNED))
 		{
-		    args[index] = SNV_UINT_TO_POINTER(va_arg(ap, unsigned));
-		}
-		else if (argtypes[index] & SNV_FLAG_SHORT)
-		{
-		    args[index] = SNV_INT_TO_POINTER(va_arg(ap, int));
-		}
-		*/
-		else if (argtypes[index] & SNV_FLAG_UNSIGNED)
-		{
-		    args[index] = SNV_UINT_TO_POINTER(va_arg(ap, unsigned));
+		    args[index] = SNV_UINT_TO_POINTER(va_arg(ap, unsigned int));
 		}
 		else
 		{
 		    args[index] = SNV_INT_TO_POINTER(va_arg(ap, int));
 		}
 		break;
+
 	    case SNV_FLOAT:
-		/* The ISO C variadic argument promotion rules obsolete
-		   this section:
-		args[index] = (snv_constpointer)snv_new(float, 1);
-		*(float*)args[index] = va_arg(ap, float);
-		break;
-		*/
 	    case SNV_DOUBLE:
 		args[index] = (snv_constpointer)snv_new(double, 1);
 		*(double*)args[index] = va_arg(ap, double);
 		break;
+
 	    case SNV_WCHAR:
-		/* The ISO C variadic argument promotion rules obsolete
-		   this section:
-		args[index] =
-		    (snv_constpointer)SNV_INT_TO_POINTER(va_arg(ap, int));
-		break;
-		*/
 	    case SNV_CHAR:
 		args[index] =
 		    (snv_constpointer)SNV_INT_TO_POINTER(va_arg(ap, int));
@@ -557,6 +535,7 @@ va_list_to_argv (format, ap)
 		break;
 	    }
 	    index++;
+#	    undef SNV_IS
 	}
     }
     
